"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.field_mask_pb2
import google.protobuf.internal.containers
import google.protobuf.message
import google.protobuf.timestamp_pb2
import sys
import typing

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class Secret(google.protobuf.message.Message):
    """API secrets allow users to make requests to the Instill AI API."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    UID_FIELD_NUMBER: builtins.int
    ID_FIELD_NUMBER: builtins.int
    CREATE_TIME_FIELD_NUMBER: builtins.int
    UPDATE_TIME_FIELD_NUMBER: builtins.int
    VALUE_FIELD_NUMBER: builtins.int
    DESCRIPTION_FIELD_NUMBER: builtins.int
    name: builtins.str
    """The name of the secret, define by its ID.
    - Format: `secrets/{secret.id}`.
    """
    uid: builtins.str
    """Secret UUID."""
    id: builtins.str
    """Secret resource ID (used in `name` as the last segment). This conforms
    to RFC-1034, which restricts to letters, numbers, and hyphen, with the
    first character a letter, the last a letter or a number, and a 63
    character maximum.
    """
    @property
    def create_time(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """Creation time."""
    @property
    def update_time(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """Update time."""
    value: builtins.str
    """The value of the secret, which is input-only and will never be returned in API responses."""
    description: builtins.str
    """Description"""
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        uid: builtins.str = ...,
        id: builtins.str = ...,
        create_time: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        update_time: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        value: builtins.str | None = ...,
        description: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["_value", b"_value", "create_time", b"create_time", "update_time", b"update_time", "value", b"value"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["_value", b"_value", "create_time", b"create_time", "description", b"description", "id", b"id", "name", b"name", "uid", b"uid", "update_time", b"update_time", "value", b"value"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["_value", b"_value"]) -> typing_extensions.Literal["value"] | None: ...

global___Secret = Secret

@typing_extensions.final
class CreateNamespaceSecretRequest(google.protobuf.message.Message):
    """CreateamespaceSecretRequest represents a request to create a secret."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAMESPACE_ID_FIELD_NUMBER: builtins.int
    SECRET_FIELD_NUMBER: builtins.int
    namespace_id: builtins.str
    """Namespace ID"""
    @property
    def secret(self) -> global___Secret:
        """The properties of the secret to be created."""
    def __init__(
        self,
        *,
        namespace_id: builtins.str = ...,
        secret: global___Secret | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["secret", b"secret"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["namespace_id", b"namespace_id", "secret", b"secret"]) -> None: ...

global___CreateNamespaceSecretRequest = CreateNamespaceSecretRequest

@typing_extensions.final
class CreateNamespaceSecretResponse(google.protobuf.message.Message):
    """CreateNamespaceSecretResponse contains the created secret."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SECRET_FIELD_NUMBER: builtins.int
    @property
    def secret(self) -> global___Secret:
        """The created secret resource."""
    def __init__(
        self,
        *,
        secret: global___Secret | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["secret", b"secret"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["secret", b"secret"]) -> None: ...

global___CreateNamespaceSecretResponse = CreateNamespaceSecretResponse

@typing_extensions.final
class ListNamespaceSecretsRequest(google.protobuf.message.Message):
    """ListNamespaceSecretsRequest represents a request to list the secrets of a namespace."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAMESPACE_ID_FIELD_NUMBER: builtins.int
    PAGE_SIZE_FIELD_NUMBER: builtins.int
    PAGE_TOKEN_FIELD_NUMBER: builtins.int
    namespace_id: builtins.str
    """Namespace ID"""
    page_size: builtins.int
    """The maximum number of secrets to return. If this parameter is unspecified,
    at most 10 pipelines will be returned. The cap value for this parameter is
    100 (i.e. any value above that will be coerced to 100).
    """
    page_token: builtins.str
    """Page secret."""
    def __init__(
        self,
        *,
        namespace_id: builtins.str = ...,
        page_size: builtins.int | None = ...,
        page_token: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["_page_size", b"_page_size", "_page_token", b"_page_token", "page_size", b"page_size", "page_token", b"page_token"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["_page_size", b"_page_size", "_page_token", b"_page_token", "namespace_id", b"namespace_id", "page_size", b"page_size", "page_token", b"page_token"]) -> None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing_extensions.Literal["_page_size", b"_page_size"]) -> typing_extensions.Literal["page_size"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing_extensions.Literal["_page_token", b"_page_token"]) -> typing_extensions.Literal["page_token"] | None: ...

global___ListNamespaceSecretsRequest = ListNamespaceSecretsRequest

@typing_extensions.final
class ListNamespaceSecretsResponse(google.protobuf.message.Message):
    """ListNamespaceSecretsResponse contains a list of secrets."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SECRETS_FIELD_NUMBER: builtins.int
    NEXT_PAGE_TOKEN_FIELD_NUMBER: builtins.int
    TOTAL_SIZE_FIELD_NUMBER: builtins.int
    @property
    def secrets(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Secret]:
        """A list of secret resources."""
    next_page_token: builtins.str
    """Next page secret."""
    total_size: builtins.int
    """Total number of secret resources."""
    def __init__(
        self,
        *,
        secrets: collections.abc.Iterable[global___Secret] | None = ...,
        next_page_token: builtins.str = ...,
        total_size: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["next_page_token", b"next_page_token", "secrets", b"secrets", "total_size", b"total_size"]) -> None: ...

global___ListNamespaceSecretsResponse = ListNamespaceSecretsResponse

@typing_extensions.final
class GetNamespaceSecretRequest(google.protobuf.message.Message):
    """GetNamespaceSecretRequest represents a request to fetch the details of a secret"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAMESPACE_ID_FIELD_NUMBER: builtins.int
    SECRET_ID_FIELD_NUMBER: builtins.int
    namespace_id: builtins.str
    """Namespace ID"""
    secret_id: builtins.str
    """Secret ID"""
    def __init__(
        self,
        *,
        namespace_id: builtins.str = ...,
        secret_id: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["namespace_id", b"namespace_id", "secret_id", b"secret_id"]) -> None: ...

global___GetNamespaceSecretRequest = GetNamespaceSecretRequest

@typing_extensions.final
class GetNamespaceSecretResponse(google.protobuf.message.Message):
    """GetNamespaceSecretResponse contains the requested secret."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SECRET_FIELD_NUMBER: builtins.int
    @property
    def secret(self) -> global___Secret:
        """The secret resource."""
    def __init__(
        self,
        *,
        secret: global___Secret | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["secret", b"secret"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["secret", b"secret"]) -> None: ...

global___GetNamespaceSecretResponse = GetNamespaceSecretResponse

@typing_extensions.final
class UpdateNamespaceSecretRequest(google.protobuf.message.Message):
    """UpdateNamespaceSecretRequest represents a request to update a namespace secret."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAMESPACE_ID_FIELD_NUMBER: builtins.int
    SECRET_ID_FIELD_NUMBER: builtins.int
    SECRET_FIELD_NUMBER: builtins.int
    UPDATE_MASK_FIELD_NUMBER: builtins.int
    namespace_id: builtins.str
    """Namespace ID"""
    secret_id: builtins.str
    """Secret ID"""
    @property
    def secret(self) -> global___Secret:
        """The secret fields to update."""
    @property
    def update_mask(self) -> google.protobuf.field_mask_pb2.FieldMask:
        """The update mask specifies the subset of fields that should be modified.

        For more information about this field, see
        https://developers.google.com/protocol-buffers/docs/reference/google.protobuf#field-mask.
        """
    def __init__(
        self,
        *,
        namespace_id: builtins.str = ...,
        secret_id: builtins.str = ...,
        secret: global___Secret | None = ...,
        update_mask: google.protobuf.field_mask_pb2.FieldMask | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["secret", b"secret", "update_mask", b"update_mask"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["namespace_id", b"namespace_id", "secret", b"secret", "secret_id", b"secret_id", "update_mask", b"update_mask"]) -> None: ...

global___UpdateNamespaceSecretRequest = UpdateNamespaceSecretRequest

@typing_extensions.final
class UpdateNamespaceSecretResponse(google.protobuf.message.Message):
    """UpdateNamespaceSecretResponse contains the updated secret."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SECRET_FIELD_NUMBER: builtins.int
    @property
    def secret(self) -> global___Secret:
        """The updated secret resource."""
    def __init__(
        self,
        *,
        secret: global___Secret | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["secret", b"secret"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["secret", b"secret"]) -> None: ...

global___UpdateNamespaceSecretResponse = UpdateNamespaceSecretResponse

@typing_extensions.final
class DeleteNamespaceSecretRequest(google.protobuf.message.Message):
    """DeleteNamespaceSecretRequest represents a request to delete a secret resource."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAMESPACE_ID_FIELD_NUMBER: builtins.int
    SECRET_ID_FIELD_NUMBER: builtins.int
    namespace_id: builtins.str
    """Namespace ID"""
    secret_id: builtins.str
    """Secret ID"""
    def __init__(
        self,
        *,
        namespace_id: builtins.str = ...,
        secret_id: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["namespace_id", b"namespace_id", "secret_id", b"secret_id"]) -> None: ...

global___DeleteNamespaceSecretRequest = DeleteNamespaceSecretRequest

@typing_extensions.final
class DeleteNamespaceSecretResponse(google.protobuf.message.Message):
    """DeleteNamespaceSecretResponse is an empty response."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___DeleteNamespaceSecretResponse = DeleteNamespaceSecretResponse

@typing_extensions.final
class CreateUserSecretRequest(google.protobuf.message.Message):
    """CreateUserSecretRequest represents a request to create a secret."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SECRET_FIELD_NUMBER: builtins.int
    PARENT_FIELD_NUMBER: builtins.int
    @property
    def secret(self) -> global___Secret:
        """The properties of the secret to be created."""
    parent: builtins.str
    """The parent resource, i.e., the user that creates the secret.
    - Format: `users/{user.id}`.
    """
    def __init__(
        self,
        *,
        secret: global___Secret | None = ...,
        parent: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["secret", b"secret"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["parent", b"parent", "secret", b"secret"]) -> None: ...

global___CreateUserSecretRequest = CreateUserSecretRequest

@typing_extensions.final
class CreateUserSecretResponse(google.protobuf.message.Message):
    """CreateUserSecretResponse contains the created secret."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SECRET_FIELD_NUMBER: builtins.int
    @property
    def secret(self) -> global___Secret:
        """The created secret resource."""
    def __init__(
        self,
        *,
        secret: global___Secret | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["secret", b"secret"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["secret", b"secret"]) -> None: ...

global___CreateUserSecretResponse = CreateUserSecretResponse

@typing_extensions.final
class ListUserSecretsRequest(google.protobuf.message.Message):
    """ListUserSecretsRequest represents a request to list the secrets of a user."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PAGE_SIZE_FIELD_NUMBER: builtins.int
    PAGE_TOKEN_FIELD_NUMBER: builtins.int
    PARENT_FIELD_NUMBER: builtins.int
    page_size: builtins.int
    """The maximum number of secrets to return. If this parameter is unspecified,
    at most 10 pipelines will be returned. The cap value for this parameter is
    100 (i.e. any value above that will be coerced to 100).
    """
    page_token: builtins.str
    """Page secret."""
    parent: builtins.str
    """The parent resource, i.e., the user that creates the secret.
    - Format: `users/{user.id}`.
    """
    def __init__(
        self,
        *,
        page_size: builtins.int | None = ...,
        page_token: builtins.str | None = ...,
        parent: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["_page_size", b"_page_size", "_page_token", b"_page_token", "page_size", b"page_size", "page_token", b"page_token"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["_page_size", b"_page_size", "_page_token", b"_page_token", "page_size", b"page_size", "page_token", b"page_token", "parent", b"parent"]) -> None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing_extensions.Literal["_page_size", b"_page_size"]) -> typing_extensions.Literal["page_size"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing_extensions.Literal["_page_token", b"_page_token"]) -> typing_extensions.Literal["page_token"] | None: ...

global___ListUserSecretsRequest = ListUserSecretsRequest

@typing_extensions.final
class ListUserSecretsResponse(google.protobuf.message.Message):
    """ListUserSecretsResponse contains a list of secrets."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SECRETS_FIELD_NUMBER: builtins.int
    NEXT_PAGE_TOKEN_FIELD_NUMBER: builtins.int
    TOTAL_SIZE_FIELD_NUMBER: builtins.int
    @property
    def secrets(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Secret]:
        """A list of secret resources."""
    next_page_token: builtins.str
    """Next page secret."""
    total_size: builtins.int
    """Total number of secret resources."""
    def __init__(
        self,
        *,
        secrets: collections.abc.Iterable[global___Secret] | None = ...,
        next_page_token: builtins.str = ...,
        total_size: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["next_page_token", b"next_page_token", "secrets", b"secrets", "total_size", b"total_size"]) -> None: ...

global___ListUserSecretsResponse = ListUserSecretsResponse

@typing_extensions.final
class GetUserSecretRequest(google.protobuf.message.Message):
    """GetUserSecretRequest represents a request to fetch the details of a secret"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    name: builtins.str
    """The resource name of the secret, which allows its access by ID.
    - Format: `users/{user.id}/secrets/{secret.id}`.
    """
    def __init__(
        self,
        *,
        name: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["name", b"name"]) -> None: ...

global___GetUserSecretRequest = GetUserSecretRequest

@typing_extensions.final
class GetUserSecretResponse(google.protobuf.message.Message):
    """GetUserSecretResponse contains the requested secret."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SECRET_FIELD_NUMBER: builtins.int
    @property
    def secret(self) -> global___Secret:
        """The secret resource."""
    def __init__(
        self,
        *,
        secret: global___Secret | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["secret", b"secret"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["secret", b"secret"]) -> None: ...

global___GetUserSecretResponse = GetUserSecretResponse

@typing_extensions.final
class UpdateUserSecretRequest(google.protobuf.message.Message):
    """UpdateUserSecretRequest represents a request to update a user secret."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SECRET_FIELD_NUMBER: builtins.int
    UPDATE_MASK_FIELD_NUMBER: builtins.int
    @property
    def secret(self) -> global___Secret:
        """The secret fields to update."""
    @property
    def update_mask(self) -> google.protobuf.field_mask_pb2.FieldMask:
        """The update mask specifies the subset of fields that should be modified.

        For more information about this field, see
        https://developers.google.com/protocol-buffers/docs/reference/google.protobuf#field-mask.
        """
    def __init__(
        self,
        *,
        secret: global___Secret | None = ...,
        update_mask: google.protobuf.field_mask_pb2.FieldMask | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["secret", b"secret", "update_mask", b"update_mask"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["secret", b"secret", "update_mask", b"update_mask"]) -> None: ...

global___UpdateUserSecretRequest = UpdateUserSecretRequest

@typing_extensions.final
class UpdateUserSecretResponse(google.protobuf.message.Message):
    """UpdateUserSecretResponse contains the updated secret."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SECRET_FIELD_NUMBER: builtins.int
    @property
    def secret(self) -> global___Secret:
        """The updated secret resource."""
    def __init__(
        self,
        *,
        secret: global___Secret | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["secret", b"secret"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["secret", b"secret"]) -> None: ...

global___UpdateUserSecretResponse = UpdateUserSecretResponse

@typing_extensions.final
class DeleteUserSecretRequest(google.protobuf.message.Message):
    """DeleteUserSecretRequest represents a request to delete a secret resource."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    name: builtins.str
    """The resource name of the secret, which allows its access by ID.
    - Format: `users/{user.id}/secrets/{secret.id}`.
    """
    def __init__(
        self,
        *,
        name: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["name", b"name"]) -> None: ...

global___DeleteUserSecretRequest = DeleteUserSecretRequest

@typing_extensions.final
class DeleteUserSecretResponse(google.protobuf.message.Message):
    """DeleteUserSecretResponse is an empty response."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___DeleteUserSecretResponse = DeleteUserSecretResponse

@typing_extensions.final
class CreateOrganizationSecretRequest(google.protobuf.message.Message):
    """CreateOrganizationSecretRequest represents a request to create a secret."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SECRET_FIELD_NUMBER: builtins.int
    PARENT_FIELD_NUMBER: builtins.int
    @property
    def secret(self) -> global___Secret:
        """The properties of the secret to be created."""
    parent: builtins.str
    """The parent resource, i.e., the organization that creates the secret.
    - Format: `organizations/{organization.id}`.
    """
    def __init__(
        self,
        *,
        secret: global___Secret | None = ...,
        parent: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["secret", b"secret"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["parent", b"parent", "secret", b"secret"]) -> None: ...

global___CreateOrganizationSecretRequest = CreateOrganizationSecretRequest

@typing_extensions.final
class CreateOrganizationSecretResponse(google.protobuf.message.Message):
    """CreateOrganizationSecretResponse contains the created secret."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SECRET_FIELD_NUMBER: builtins.int
    @property
    def secret(self) -> global___Secret:
        """The created secret resource."""
    def __init__(
        self,
        *,
        secret: global___Secret | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["secret", b"secret"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["secret", b"secret"]) -> None: ...

global___CreateOrganizationSecretResponse = CreateOrganizationSecretResponse

@typing_extensions.final
class ListOrganizationSecretsRequest(google.protobuf.message.Message):
    """ListOrganizationSecretsRequest represents a request to list the secrets of a user."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PAGE_SIZE_FIELD_NUMBER: builtins.int
    PAGE_TOKEN_FIELD_NUMBER: builtins.int
    PARENT_FIELD_NUMBER: builtins.int
    page_size: builtins.int
    """The maximum number of secrets to return. If this parameter is unspecified,
    at most 10 pipelines will be returned. The cap value for this parameter is
    100 (i.e. any value above that will be coerced to 100).
    """
    page_token: builtins.str
    """Page secret."""
    parent: builtins.str
    """The parent resource, i.e., the organization that creates the secret.
    - Format: `organizations/{organization.id}`.
    """
    def __init__(
        self,
        *,
        page_size: builtins.int | None = ...,
        page_token: builtins.str | None = ...,
        parent: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["_page_size", b"_page_size", "_page_token", b"_page_token", "page_size", b"page_size", "page_token", b"page_token"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["_page_size", b"_page_size", "_page_token", b"_page_token", "page_size", b"page_size", "page_token", b"page_token", "parent", b"parent"]) -> None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing_extensions.Literal["_page_size", b"_page_size"]) -> typing_extensions.Literal["page_size"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing_extensions.Literal["_page_token", b"_page_token"]) -> typing_extensions.Literal["page_token"] | None: ...

global___ListOrganizationSecretsRequest = ListOrganizationSecretsRequest

@typing_extensions.final
class ListOrganizationSecretsResponse(google.protobuf.message.Message):
    """ListOrganizationSecretsResponse contains a list of secrets."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SECRETS_FIELD_NUMBER: builtins.int
    NEXT_PAGE_TOKEN_FIELD_NUMBER: builtins.int
    TOTAL_SIZE_FIELD_NUMBER: builtins.int
    @property
    def secrets(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Secret]:
        """A list of secret resources."""
    next_page_token: builtins.str
    """Next page secret."""
    total_size: builtins.int
    """Total number of secret resources."""
    def __init__(
        self,
        *,
        secrets: collections.abc.Iterable[global___Secret] | None = ...,
        next_page_token: builtins.str = ...,
        total_size: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["next_page_token", b"next_page_token", "secrets", b"secrets", "total_size", b"total_size"]) -> None: ...

global___ListOrganizationSecretsResponse = ListOrganizationSecretsResponse

@typing_extensions.final
class GetOrganizationSecretRequest(google.protobuf.message.Message):
    """GetOrganizationSecretRequest represents a request to fetch the details of a secret"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    name: builtins.str
    """The resource name of the secret, which allows its access by ID.
    - Format: `organizations/{organization.id}/secrets/{secret.id}`.
    """
    def __init__(
        self,
        *,
        name: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["name", b"name"]) -> None: ...

global___GetOrganizationSecretRequest = GetOrganizationSecretRequest

@typing_extensions.final
class GetOrganizationSecretResponse(google.protobuf.message.Message):
    """GetOrganizationSecretResponse contains the requested secret."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SECRET_FIELD_NUMBER: builtins.int
    @property
    def secret(self) -> global___Secret:
        """The secret resource."""
    def __init__(
        self,
        *,
        secret: global___Secret | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["secret", b"secret"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["secret", b"secret"]) -> None: ...

global___GetOrganizationSecretResponse = GetOrganizationSecretResponse

@typing_extensions.final
class UpdateOrganizationSecretRequest(google.protobuf.message.Message):
    """UpdateOrganizationSecretRequest represents a request to update a user secret."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SECRET_FIELD_NUMBER: builtins.int
    UPDATE_MASK_FIELD_NUMBER: builtins.int
    @property
    def secret(self) -> global___Secret:
        """The secret fields to update."""
    @property
    def update_mask(self) -> google.protobuf.field_mask_pb2.FieldMask:
        """The update mask specifies the subset of fields that should be modified.

        For more information about this field, see
        https://developers.google.com/protocol-buffers/docs/reference/google.protobuf#field-mask.
        """
    def __init__(
        self,
        *,
        secret: global___Secret | None = ...,
        update_mask: google.protobuf.field_mask_pb2.FieldMask | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["secret", b"secret", "update_mask", b"update_mask"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["secret", b"secret", "update_mask", b"update_mask"]) -> None: ...

global___UpdateOrganizationSecretRequest = UpdateOrganizationSecretRequest

@typing_extensions.final
class UpdateOrganizationSecretResponse(google.protobuf.message.Message):
    """UpdateOrganizationSecretResponse contains the updated secret."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SECRET_FIELD_NUMBER: builtins.int
    @property
    def secret(self) -> global___Secret:
        """The updated secret resource."""
    def __init__(
        self,
        *,
        secret: global___Secret | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["secret", b"secret"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["secret", b"secret"]) -> None: ...

global___UpdateOrganizationSecretResponse = UpdateOrganizationSecretResponse

@typing_extensions.final
class DeleteOrganizationSecretRequest(google.protobuf.message.Message):
    """DeleteOrganizationSecretRequest represents a request to delete a secret resource."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    name: builtins.str
    """The resource name of the secret, which allows its access by ID.
    - Format: `organizations/{organization.id}/secrets/{secret.id}`.
    """
    def __init__(
        self,
        *,
        name: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["name", b"name"]) -> None: ...

global___DeleteOrganizationSecretRequest = DeleteOrganizationSecretRequest

@typing_extensions.final
class DeleteOrganizationSecretResponse(google.protobuf.message.Message):
    """DeleteOrganizationSecretResponse is an empty response."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___DeleteOrganizationSecretResponse = DeleteOrganizationSecretResponse



# pylint: disable=no-member,no-name-in-module, inconsistent-return-statements, unused-import
import base64
import io
import re
from typing import Dict, List, Union

import numpy as np
import requests
from google.protobuf import json_format, struct_pb2
from PIL import Image
from starlette.requests import Request

from instill.helpers.const import (
    HEADERS,
    IMAGE_INPUT_TYPE_BASE64,
    IMAGE_INPUT_TYPE_URL,
    PROMPT_ROLES,
    ChatInput,
    ChatMultiModalInput,
    CompletionInput,
    ImageEmbeddingInput,
    MultimodalEmbeddingInput,
    TextEmbeddingInput,
    TextToImageInput,
    VisionInput,
)
from instill.helpers.errors import InvalidInputException, InvalidOutputShapeException
from instill.helpers.protobufs.ray_pb2 import CallRequest, CallResponse


def base64_to_pil_image(base64_str):
    return Image.open(
        io.BytesIO(
            base64.b64decode(
                re.sub(
                    "^data:image/.+;base64,",
                    "",
                    base64_str,
                )
            )
        )
    )


def url_to_pil_image(url):
    resp = requests.get(url, headers=HEADERS, timeout=10)
    resp.raise_for_status()
    return Image.open(io.BytesIO(resp.content))


def snake_to_lower_camel(name):
    """Convert snake_case to lowerCamelCase."""
    components = name.split("_")
    return components[0] + "".join(x.title() for x in components[1:])


def dict_to_struct(dict_data):
    """Convert Dict to Struct"""
    struct_pb = struct_pb2.Struct()
    json_format.ParseDict(dict_data, struct_pb)

    return struct_pb


async def _parse_vision_task_to_vision_input(
    request: Union[CallRequest, Request],
) -> List[VisionInput]:

    def extract_data(data: dict) -> VisionInput:
        image_type = data["type"]

        inp = VisionInput()
        if image_type == IMAGE_INPUT_TYPE_URL:
            inp.image = url_to_pil_image(data[image_type])
        elif image_type == IMAGE_INPUT_TYPE_BASE64:
            inp.image = base64_to_pil_image(data[image_type])
        else:
            raise InvalidInputException("input image type not supported")

        return inp

    # http test input
    if isinstance(request, Request):
        data: dict = await request.json()

        if "type" not in data:
            data["type"] = IMAGE_INPUT_TYPE_URL

        return [extract_data(data)]

    input_list = []
    for task_input in request.task_inputs:
        task_input_dict = json_format.MessageToDict(task_input)

        data = task_input_dict["data"]

        input_list.append(extract_data(data))

    return input_list


async def parse_task_classification_to_vision_input(
    request: Union[CallRequest, Request],
) -> List[VisionInput]:

    return await _parse_vision_task_to_vision_input(request)


def construct_task_classification_output(
    request: Union[CallRequest, Request],
    categories: List[str],
    scores: List[float],
) -> Union[CallResponse, List]:
    if not len(categories) == len(scores):
        raise InvalidOutputShapeException

    task_outputs = []
    for category, score in zip(categories, scores):
        data = {"category": str(category), "score": float(score)}

        if isinstance(request, Request):
            task_outputs.append({"data": data})
        else:
            task_outputs.append(dict_to_struct({"data": data}))

    if isinstance(request, Request):
        return task_outputs

    return CallResponse(task_outputs=task_outputs)


async def parse_task_detection_to_vision_input(
    request: Union[CallRequest, Request],
) -> List[VisionInput]:

    return await _parse_vision_task_to_vision_input(request)


def construct_task_detection_output(
    request: Union[CallRequest, Request],
    categories: List[List[str]],
    scores: List[List[float]],
    bounding_boxes: List[List[tuple]],
) -> Union[CallResponse, List]:
    """Construct trigger output for detection task

    Args:
        categories (List[List[str]]): for each image input, the list of detected object's category
        scores (List[List[float]]): for each image input, the list of detected object's score
        bounding_boxes (List[List[tuple]]): for each image input, the list of detected object's bbox, with the format
        (top left x, top left y, width, height)
    """
    if not len(categories) == len(scores) == len(bounding_boxes):
        raise InvalidOutputShapeException

    task_outputs = []
    for category, score, bbox in zip(categories, scores, bounding_boxes):
        data = {}
        objects = []
        for cat, sc, bb in zip(category, score, bbox):
            bb_dict = {
                "top": float(bb[1]),
                "left": float(bb[0]),
                "width": float(bb[2]),
                "height": float(bb[3]),
            }
            objects.append(
                {"category": str(cat), "score": float(sc), "bounding-box": bb_dict}
            )

        data["objects"] = objects

        if isinstance(request, Request):
            task_outputs.append({"data": data})
        else:
            task_outputs.append(dict_to_struct({"data": data}))

    if isinstance(request, Request):
        return task_outputs

    return CallResponse(task_outputs=task_outputs)


async def parse_task_ocr_to_vision_input(
    request: Union[CallRequest, Request],
) -> List[VisionInput]:

    return await _parse_vision_task_to_vision_input(request)


def construct_task_ocr_output(
    request: Union[CallRequest, Request],
    texts: List[List[str]],
    scores: List[List[float]],
    bounding_boxes: List[List[tuple]],
) -> Union[CallResponse, List]:
    """Construct trigger output for ocr task

    Args:
        texts (List[List[str]]): for each image input, the list of detected text
        scores (List[List[float]]): for each image input, the list of detected text's score
        bounding_boxes (List[List[tuple]]): for each image input, the list of detected text's bbox, with the format
        (top left x, top left y, width, height)
    """
    if not len(texts) == len(scores) == len(bounding_boxes):
        raise InvalidOutputShapeException

    task_outputs = []
    for text, score, bbox in zip(texts, scores, bounding_boxes):
        data = {}
        objects = []
        for txt, sc, bb in zip(text, score, bbox):
            bb_dict = {
                "top": float(bb[1]),
                "left": float(bb[0]),
                "width": float(bb[2]),
                "height": float(bb[3]),
            }
            objects.append(
                {"text": str(txt), "score": float(sc), "bounding-box": bb_dict}
            )

        data["objects"] = objects

        if isinstance(request, Request):
            task_outputs.append({"data": data})
        else:
            task_outputs.append(dict_to_struct({"data": data}))

    if isinstance(request, Request):
        return task_outputs

    return CallResponse(task_outputs=task_outputs)


async def parse_task_instance_segmentation_to_vision_input(
    request: Union[CallRequest, Request],
) -> List[VisionInput]:

    return await _parse_vision_task_to_vision_input(request)


def construct_task_instance_segmentation_output(
    request: Union[CallRequest, Request],
    rles: List[List[str]],
    categories: List[List[str]],
    scores: List[List[float]],
    bounding_boxes: List[List[tuple]],
) -> Union[CallResponse, List]:
    """Construct trigger output for instance segmentation task

    Args:
        rles (List[List[str]]): for each image input, the list of detected object's rle
        categories (List[List[str]]): for each image input, the list of detected object's category
        scores (List[List[float]]): for each image input, the list of detected text's score
        bounding_boxes (List[List[tuple]]): for each image input, the list of detected text's bbox, with the format
        (top left x, top left y, width, height)
    """
    if not len(rles) == len(categories) == len(scores) == len(bounding_boxes):
        raise InvalidOutputShapeException

    task_outputs = []
    for rle, category, score, bbox in zip(rles, categories, scores, bounding_boxes):
        data = {}
        objects = []
        for r, cat, sc, bb in zip(rle, category, score, bbox):
            bb_dict = {
                "top": float(bb[1]),
                "left": float(bb[0]),
                "width": float(bb[2]),
                "height": float(bb[3]),
            }
            objects.append(
                {
                    "rle": str(r),
                    "category": str(cat),
                    "score": float(sc),
                    "bounding-box": bb_dict,
                }
            )

        data["objects"] = objects

        if isinstance(request, Request):
            task_outputs.append({"data": data})
        else:
            task_outputs.append(dict_to_struct({"data": data}))

    if isinstance(request, Request):
        return task_outputs

    return CallResponse(task_outputs=task_outputs)


async def parse_task_semantic_segmentation_to_vision_input(
    request: Union[CallRequest, Request],
) -> List[VisionInput]:

    return await _parse_vision_task_to_vision_input(request)


def construct_task_semantic_segmentation_output(
    request: Union[CallRequest, Request],
    rles: List[List[str]],
    categories: List[List[str]],
) -> Union[CallResponse, List]:
    """Construct trigger output for semantic segmentation task

    Args:
        rles (List[List[str]]): for each image input, the list of detected object's rle
        categories (List[List[str]]): for each image input, the list of detected object's category
        (top, left, width, height)
    """
    if not len(rles) == len(categories):
        raise InvalidOutputShapeException

    task_outputs = []
    for rle, category in zip(rles, categories):
        data = {}
        objects = []
        for r, cat in zip(rle, category):
            objects.append({"rle": str(r), "category": str(cat)})

        data["objects"] = objects

        if isinstance(request, Request):
            task_outputs.append({"data": data})
        else:
            task_outputs.append(dict_to_struct({"data": data}))

    if isinstance(request, Request):
        return task_outputs

    return CallResponse(task_outputs=task_outputs)


async def parse_task_keypoint_to_vision_input(
    request: Union[CallRequest, Request],
) -> List[VisionInput]:

    return await _parse_vision_task_to_vision_input(request)


def construct_task_keypoint_output(
    request: Union[CallRequest, Request],
    keypoints: List[List[List[tuple]]],
    scores: List[List[float]],
    bounding_boxes: List[List[tuple]],
) -> Union[CallResponse, List]:
    """Construct trigger output for keypoint task

    Args:
        keypoints (List[List[List[str]]]): for each image input, the list of detected object's keypoints,
        with the format (x_coordinate, y_coordinate, visibility)
        scores (List[List[float]]): for each image input, the list of detected object's score
        bounding_boxes (List[List[tuple]]): for each image input, the list of detected object's bbox, with the format
        (top left x, top left y, width, height)
    """

    if not len(keypoints) == len(scores) == len(bounding_boxes):
        raise InvalidOutputShapeException

    task_outputs = []
    for keypoint, score, bbox in zip(keypoints, scores, bounding_boxes):
        data = {}
        objects = []
        for kps, sc, bb in zip(keypoint, score, bbox):
            point_list = []
            for kp in kps:
                point_list.append({"x": kp[0], "y": kp[1], "v": kp[2]})

            bb_dict = {
                "top": float(bb[1]),
                "left": float(bb[0]),
                "width": float(bb[2]),
                "height": float(bb[3]),
            }
            objects.append(
                {"keypoints": point_list, "score": float(sc), "bounding-box": bb_dict}
            )

        data["objects"] = objects

        if isinstance(request, Request):
            task_outputs.append({"data": data})
        else:
            task_outputs.append(dict_to_struct({"data": data}))

    if isinstance(request, Request):
        return task_outputs

    return CallResponse(task_outputs=task_outputs)


async def parse_task_completion_to_completion_input(
    request: Union[CallRequest, Request],
) -> List[CompletionInput]:

    # http test input
    if isinstance(request, Request):
        test_data: dict = await request.json()

        inp = CompletionInput()
        inp.prompt = test_data["prompt"]
        return [inp]

    input_list = []
    for task_input in request.task_inputs:
        task_input_dict = json_format.MessageToDict(task_input)

        data = task_input_dict["data"]
        parameter = (
            task_input_dict["parameter"] if "parameter" in task_input_dict else {}
        )

        inp = CompletionInput()

        inp.prompt = data["prompt"]

        # system message
        if "system-message" in data:
            inp.system_message = data["system-message"]
        else:
            inp.system_message = (
                "You are a helpful, respectful and honest assistant. "
                "Always answer as helpfully as possible, while being safe.  "
                "Your answers should not include any harmful, unethical, racist, "
                "sexist, toxic, dangerous, or illegal content. Please ensure that "
                "your responses are socially unbiased and positive in nature. "
                "If a question does not make any sense, or is not factually coherent, "
                "explain why instead of answering something not correct. If you don't "
                "know the answer to a question, please don't share false information."
            )

        # max tokens
        if "max-tokens" in parameter:
            inp.max_tokens = int(parameter["max-tokens"])

        # temperature
        if "temperature" in parameter:
            inp.temperature = parameter["temperature"]

        # number of generated outputs
        if "n" in parameter:
            inp.n = int(parameter["n"])

        # top p
        if "top-p" in parameter:
            inp.top_p = int(parameter["top-p"])

        # seed
        if "seed" in parameter:
            inp.seed = int(parameter["seed"])

        # stream
        if "stream" in parameter:
            inp.seed = int(parameter["stream"])

        input_list.append(inp)

    return input_list


def construct_task_completion_output(
    request: Union[CallRequest, Request],
    finish_reasons: List[List[str]],
    indexes: List[List[int]],
    created_timestamps: List[List[int]],
    contents: List[List[str]],
    completion_tokens: Union[List[int], None] = None,
    prompt_tokens: Union[List[int], None] = None,
    total_tokens: Union[List[int], None] = None,
) -> Union[CallResponse, List]:

    if (
        not len(finish_reasons)
        == len(indexes)
        == len(created_timestamps)
        == len(contents)
    ):
        raise InvalidOutputShapeException
    if len(contents) > 0 and not (
        len(finish_reasons[0])
        == len(indexes[0])
        == len(created_timestamps[0])
        == len(contents[0])
    ):
        raise InvalidOutputShapeException

    if completion_tokens is None or prompt_tokens is None or total_tokens is None:
        print("one or more of token usage number not set, ignore all ")
    else:
        if not len(completion_tokens) == len(prompt_tokens) == len(total_tokens):
            raise InvalidOutputShapeException

    task_outputs = []
    # data
    for finish_reason_list, index_list, created_timestamp_list, content_list in zip(
        finish_reasons, indexes, created_timestamps, contents
    ):
        data = {}
        choices = []
        for (
            finish_reason,
            index,
            created_timestamp,
            content,
        ) in zip(finish_reason_list, index_list, created_timestamp_list, content_list):
            choices.append(
                {
                    "finish-reason": finish_reason,
                    "index": index,
                    "content": content,
                    "created": created_timestamp,
                }
            )
        data["choices"] = choices
        task_outputs.append({"data": data})

    # metadata
    if (
        completion_tokens is not None
        and prompt_tokens is not None
        and total_tokens is not None
    ):
        for i, (completion_token, prompt_token, total_token) in enumerate(
            zip(completion_tokens, prompt_tokens, total_tokens)
        ):
            metadata = {
                "usage": {
                    "completion-tokens": completion_token,
                    "prompt-tokens": prompt_token,
                    "total-tokens": total_token,
                }
            }
            task_outputs[i]["metadata"] = metadata  # type: ignore

    if isinstance(request, Request):
        return task_outputs

    for i, o in enumerate(task_outputs):
        task_outputs[i] = dict_to_struct(o)

    return CallResponse(task_outputs=task_outputs)


async def parse_task_chat_to_chat_input(
    request: Union[CallRequest, Request],
) -> List[ChatInput]:

    # http test input
    if isinstance(request, Request):
        test_data: dict = await request.json()

        inp = ChatInput()
        inp.messages = [{"role": "user", "content": test_data["prompt"]}]
        return [inp]

    input_list = []
    for task_input in request.task_inputs:
        task_input_dict = json_format.MessageToDict(task_input)

        data = task_input_dict["data"]
        parameter = (
            task_input_dict["parameter"] if "parameter" in task_input_dict else {}
        )

        inp = ChatInput()

        messages: List[Dict[str, str]] = []

        # messages
        if len(data["messages"]) > 0:
            for chat_entity in data["messages"]:
                chat_message = None
                if len(chat_entity["content"]) > 1:
                    raise ValueError(
                        "Multiple text message detected"
                        " in a single chat history entity"
                    )

                if "text" in chat_entity["content"][0]:
                    chat_message = chat_entity["content"][0]["text"]
                else:
                    raise ValueError(
                        "Unsupported chat_hisotry message type"
                        ", expected 'text'"
                        f" but get {chat_entity['content'][0]}"
                    )

                if chat_entity["role"] not in PROMPT_ROLES:
                    raise ValueError(
                        f"Role `{chat_entity['role']}` is not in supported"
                        f"role list ({','.join(PROMPT_ROLES)})"
                    )
                if chat_message is None:
                    raise ValueError(
                        f"No message found in chat_history. {chat_message}"
                    )

                if (
                    len(messages) > 0
                    and messages[-1]["role"] != PROMPT_ROLES[0]
                    and chat_entity["role"] != PROMPT_ROLES[0]
                ):
                    messages.append({"role": "user", "content": " "})

                if len(messages) > 0 and messages[-1]["role"] == chat_entity["role"]:
                    last_conversation = messages.pop()
                    chat_message = f"{last_conversation['content']}\n\n{chat_message}"

                messages.append({"role": chat_entity["role"], "content": chat_message})

            if messages[0]["role"] != PROMPT_ROLES[-1]:
                messages.insert(
                    0,
                    {
                        "role": PROMPT_ROLES[-1],
                        "content": (
                            "You are a helpful, respectful and honest assistant. "
                            "Always answer as helpfully as possible, while being safe.  "
                            "Your answers should not include any harmful, unethical, racist, "
                            "sexist, toxic, dangerous, or illegal content. Please ensure that "
                            "your responses are socially unbiased and positive in nature. "
                            "If a question does not make any sense, or is not factually coherent, "
                            "explain why instead of answering something not correct. If you don't "
                            "know the answer to a question, please don't share false information."
                        ),
                    },
                )

        inp.messages = messages

        # max tokens
        if "max-tokens" in parameter:
            inp.max_tokens = int(parameter["max-tokens"])

        # temperature
        if "temperature" in parameter:
            inp.temperature = parameter["temperature"]

        # number of generated outputs
        if "n" in parameter:
            inp.n = int(parameter["n"])

        # top p
        if "top-p" in parameter:
            inp.top_p = int(parameter["top-p"])

        # seed
        if "seed" in parameter:
            inp.seed = int(parameter["seed"])

        # stream
        if "stream" in parameter:
            inp.seed = int(parameter["stream"])

        input_list.append(inp)

    return input_list


def construct_task_chat_output(
    request: Union[CallRequest, Request],
    finish_reasons: List[List[str]],
    indexes: List[List[int]],
    created_timestamps: List[List[int]],
    messages: List[List[dict]],
    completion_tokens: Union[List[int], None] = None,
    prompt_tokens: Union[List[int], None] = None,
    total_tokens: Union[List[int], None] = None,
) -> Union[CallResponse, List]:

    if (
        not len(finish_reasons)
        == len(indexes)
        == len(created_timestamps)
        == len(messages)
    ):
        raise InvalidOutputShapeException
    if len(messages) > 0 and not (
        len(finish_reasons[0])
        == len(indexes[0])
        == len(created_timestamps[0])
        == len(messages[0])
    ):
        raise InvalidOutputShapeException

    if completion_tokens is None or prompt_tokens is None or total_tokens is None:
        print("one or more of token usage number not set, ignore all ")
    else:
        if not len(completion_tokens) == len(prompt_tokens) == len(total_tokens):
            raise InvalidOutputShapeException

    task_outputs = []
    # data
    for finish_reason_list, index_list, created_timestamp_list, message_list in zip(
        finish_reasons, indexes, created_timestamps, messages
    ):
        data = {}
        choices = []
        for (
            finish_reason,
            index,
            created_timestamp,
            message,
        ) in zip(finish_reason_list, index_list, created_timestamp_list, message_list):
            choices.append(
                {
                    "finish-reason": finish_reason,
                    "index": index,
                    "message": message,
                    "created": created_timestamp,
                }
            )
        data["choices"] = choices
        task_outputs.append({"data": data})

    # metadata
    if (
        completion_tokens is not None
        and prompt_tokens is not None
        and total_tokens is not None
    ):
        for i, (completion_token, prompt_token, total_token) in enumerate(
            zip(completion_tokens, prompt_tokens, total_tokens)
        ):
            metadata = {
                "usage": {
                    "completion-tokens": completion_token,
                    "prompt-tokens": prompt_token,
                    "total-tokens": total_token,
                }
            }
            task_outputs[i]["metadata"] = metadata  # type: ignore

    if isinstance(request, Request):
        return task_outputs

    for i, o in enumerate(task_outputs):
        task_outputs[i] = dict_to_struct(o)

    return CallResponse(task_outputs=task_outputs)


async def parse_task_chat_to_multimodal_chat_input(
    request: Union[CallRequest, Request],
) -> List[ChatMultiModalInput]:

    # http test input
    if isinstance(request, Request):
        test_data: dict = await request.json()

        test_prompt = test_data["prompt"]
        image_url = test_data["image-url"]

        inp = ChatMultiModalInput()
        inp.messages = [
            {
                "role": "user",
                "content": test_prompt,
            }
        ]
        inp.prompt_images = [[url_to_pil_image(image_url)]]
        return [inp]

    input_list = []
    for task_input in request.task_inputs:
        task_input_dict = json_format.MessageToDict(task_input)

        data = task_input_dict["data"]
        parameter = (
            task_input_dict["parameter"] if "parameter" in task_input_dict else {}
        )

        inp = ChatMultiModalInput()

        # messages and prompt images
        messages: List[Dict[str, str]] = []
        images: List[List[Image.Image]] = []
        for i, message in enumerate(data["messages"]):
            role = message["role"]
            content = message["content"]

            imgs = []
            if role == PROMPT_ROLES[0]:
                for c in content:
                    if c["type"] == "text":
                        if len(messages) > i:
                            raise InvalidInputException(
                                "can only have single text from user in each round"
                            )
                        messages.insert(i, {"role": role, "content": c["text"]})
                    elif c["type"] == "image-url":
                        imgs.append(url_to_pil_image(c["image-url"]))
                    elif c["type"] == "image-base64":
                        imgs.append(base64_to_pil_image(c["image-base64"]))
                    else:
                        raise InvalidInputException("input content type not supported")
            elif role == PROMPT_ROLES[1]:
                messages.append({"role": role, "content": content[0]["text"]})
            elif role == PROMPT_ROLES[-1] and messages[0]["role"] != PROMPT_ROLES[-1]:
                messages.insert(0, {"role": role, "content": content[0]["text"]})

            images.append(imgs)

        if messages[0]["role"] != PROMPT_ROLES[-1]:
            images.insert(0, [])
            messages.insert(
                0,
                {
                    "role": PROMPT_ROLES[-1],
                    "content": (
                        "You are a helpful, respectful and honest assistant. "
                        "Always answer as helpfully as possible, while being safe.  "
                        "Your answers should not include any harmful, unethical, racist, "
                        "sexist, toxic, dangerous, or illegal content. Please ensure that "
                        "your responses are socially unbiased and positive in nature. "
                        "If a question does not make any sense, or is not factually coherent, "
                        "explain why instead of answering something not correct. If you don't "
                        "know the answer to a question, please don't share false information."
                    ),
                },
            )

        inp.messages = messages
        inp.prompt_images = images

        # max tokens
        if "max-tokens" in parameter:
            inp.max_tokens = int(parameter["max-tokens"])

        # temperature
        if "temperature" in parameter:
            inp.temperature = parameter["temperature"]

        # number of generated outputs
        if "n" in parameter:
            inp.n = int(parameter["n"])

        # top p
        if "top-p" in parameter:
            inp.top_p = int(parameter["top-p"])

        # seed
        if "seed" in parameter:
            inp.seed = int(parameter["seed"])

        # stream
        if "stream" in parameter:
            inp.seed = int(parameter["stream"])

        input_list.append(inp)

    return input_list


async def parse_task_text_to_image_input(
    request: Union[CallRequest, Request],
) -> List[TextToImageInput]:

    # http test input
    if isinstance(request, Request):
        data: dict = await request.json()

        inp = TextToImageInput()
        inp.prompt = data["prompt"]

        return [inp]

    input_list = []
    for task_input in request.task_inputs:
        task_input_dict = json_format.MessageToDict(task_input)

        data = task_input_dict["data"]
        parameter = (
            task_input_dict["parameter"] if "parameter" in task_input_dict else {}
        )

        inp = TextToImageInput()

        # prompt
        inp.prompt = data["prompt"]

        # negative prompt
        if "negative-prompt" in parameter:
            inp.negative_prompt = str(parameter["negative-prompt"])

        # aspect ratio
        if "aspect-ratio" in parameter:
            inp.aspect_ratio = str(parameter["aspect-ratio"])

        # number of generated outputs
        if "n" in parameter:
            inp.n = int(parameter["n"])

        # seed
        if "seed" in parameter:
            inp.seed = int(parameter["seed"])

        input_list.append(inp)

    return input_list


def construct_task_text_to_image_output(
    request: Union[CallRequest, Request],
    finish_reasons: List[List[str]],
    images: List[List[str]],
) -> Union[CallResponse, List]:
    """Construct trigger output for text to image task

    Args:
        images (List[List[str]]): for each input prompt,
        the generated jpeg image base64 string with the length of `samples`
    """

    if not len(finish_reasons) == len(images):
        raise InvalidOutputShapeException
    if len(images) > 0 and not len(finish_reasons[0]) == len(images[0]):
        raise InvalidOutputShapeException

    task_outputs = []
    for imgs, finishes in zip(images, finish_reasons):
        data = {}
        choices = []

        for img, finish in zip(imgs, finishes):
            choices.append(
                {
                    "finish-reason": finish,
                    "image": f"data:image/jpeg;base64,{img}",
                }
            )
        data["choices"] = choices
        task_outputs.append({"data": data})

    if isinstance(request, Request):
        return task_outputs

    for i, o in enumerate(task_outputs):
        task_outputs[i] = dict_to_struct(o)

    return CallResponse(task_outputs=task_outputs)


# async def parse_task_image_to_image_input(
#     request: Union[CallRequest, Request],
# ) -> List[ImageToImageInput]:

#     # http test input
#     if isinstance(request, Request):
#         data: dict = await request.json()

#         test_prompt = data["prompt"]
#         test_image_url = data["image_url"]

#         inp = ImageToImageInput()
#         inp.prompt = test_prompt
#         inp.prompt_image = url_to_pil_image(test_image_url)

#         return [inp]

#     input_list = []
#     for task_input in request.task_inputs:
#         task_input_dict = json_format.MessageToDict(task_input)["ImageToImage"]

#         inp = ImageToImageInput()

#         # prompt
#         inp.prompt = task_input_dict["prompt"]

#         # prompt images
#         if (
#             "Promptimage-url" in task_input_dict["Type"]
#             and "Promptimage-base64" in task_input_dict["Type"]
#         ) or (
#             "Promptimage-url" not in task_input_dict["Type"]
#             and "Promptimage-base64" not in task_input_dict["Type"]
#         ):
#             raise InvalidInputException
#         if "Promptimage-url" in task_input_dict["Type"]:
#             inp.prompt_image = url_to_pil_image(
#                 task_input_dict["Type"]["Promptimage-url"]
#             )
#         elif "Promptimage-base64" in task_input_dict["Type"]:
#             inp.prompt_image = base64_to_pil_image(
#                 task_input_dict["Type"]["Promptimage-base64"]
#             )

#         # steps
#         if "steps" in task_input_dict:
#             inp.steps = int(task_input_dict["steps"])

#         # cfg_scale
#         if "cfg_scale" in task_input_dict:
#             inp.cfg_scale = task_input_dict["cfg_scale"]

#         # samples
#         if "samples" in task_input_dict:
#             inp.samples = int(task_input_dict["samples"])

#         # seed
#         if "seed" in task_input_dict:
#             inp.seed = int(task_input_dict["seed"])

#         input_list.append(inp)

#     return input_list


# def construct_task_image_to_image_output(
#     request: Union[CallRequest, Request],
#     images: List[List[str]],
# ) -> Union[CallResponse, List[List[str]]]:
#     """Construct trigger output for keypoint task

#     Args:
#         images (List[List[str]]): for each input prompt, the generated images with the length of `samples`
#     """

#     if isinstance(request, Request):
#         return images

#     task_outputs = []
#     for imgs in images:
#         task_outputs.append(
#             protobuf_to_struct(
#                 modelpb.TaskOutput(
#                     image_to_image=imagetoimagepb.ImageToImageOutput(images=imgs)
#                 )
#             )
#         )

#     return CallResponse(task_outputs=task_outputs)


async def parse_task_embedding_to_text_embedding_input(
    request: Union[CallRequest, Request],
) -> List[TextEmbeddingInput]:

    # http test input
    if isinstance(request, Request):
        test_data: dict = await request.json()

        test_prompt = test_data["prompt"]

        inp = TextEmbeddingInput()
        inp.contents = [test_prompt]

        return [inp]

    input_list = []
    for task_input in request.task_inputs:
        task_input_dict = json_format.MessageToDict(task_input)

        data = task_input_dict["data"]
        parameter = (
            task_input_dict["parameter"] if "parameter" in task_input_dict else {}
        )

        inp = TextEmbeddingInput()

        # messages and prompt images
        contents: List[str] = []
        for embedding in data["embeddings"]:
            if embedding["type"] == "text":
                contents.append(embedding["text"])
            else:
                raise InvalidInputException("this model can only process text input")

        inp.contents = contents

        # format
        if "format" in parameter:
            inp.format = str(parameter["format"])

        # dimensions
        if "dimensions" in parameter:
            inp.dimensions = int(parameter["dimensions"])

        # input-type
        if "input-type" in parameter:
            inp.input_type = str(parameter["input-type"])

        # truncate
        if "truncate" in parameter:
            inp.truncate = str(parameter["truncate"])

        input_list.append(inp)

    return input_list


async def parse_task_embedding_to_image_embedding_input(
    request: Union[CallRequest, Request],
) -> List[ImageEmbeddingInput]:

    # http test input
    if isinstance(request, Request):
        test_data: dict = await request.json()

        test_img = test_data["image"]

        inp = ImageEmbeddingInput()
        inp.images = [url_to_pil_image(test_img)]

        return [inp]

    input_list = []
    for task_input in request.task_inputs:
        task_input_dict = json_format.MessageToDict(task_input)

        data = task_input_dict["data"]
        parameter = (
            task_input_dict["parameter"] if "parameter" in task_input_dict else {}
        )

        inp = ImageEmbeddingInput()

        # messages and prompt images
        images: List[Image.Image] = []
        for embedding in data["embeddings"]:
            if embedding["type"] == "image-url":
                images.append(url_to_pil_image(embedding[embedding["type"]]))
            elif embedding["type"] == "image-base64":
                images.append(base64_to_pil_image(embedding[embedding["type"]]))
            else:
                raise InvalidInputException("can only process image input")

        inp.images = images

        # format
        if "format" in parameter:
            inp.format = str(parameter["format"])

        # dimensions
        if "dimensions" in parameter:
            inp.dimensions = int(parameter["dimensions"])

        # input-type
        if "input-type" in parameter:
            inp.input_type = str(parameter["input-type"])

        # truncate
        if "truncate" in parameter:
            inp.truncate = str(parameter["truncate"])

        input_list.append(inp)

    return input_list


async def parse_task_embedding_to_multimodal_embedding_input(
    request: Union[CallRequest, Request],
) -> List[MultimodalEmbeddingInput]:

    # http test input
    if isinstance(request, Request):
        test_data: dict = await request.json()

        inp = MultimodalEmbeddingInput()
        inp.contents = []
        if "image" in test_data:
            inp.contents.append(
                {"type": "image", "image": url_to_pil_image(test_data["image"])}
            )
        if "text" in test_data:
            inp.contents.append({"type": "text", "text": test_data["text"]})

        return [inp]

    input_list = []
    for task_input in request.task_inputs:
        task_input_dict = json_format.MessageToDict(task_input)

        data = task_input_dict["data"]
        parameter = (
            task_input_dict["parameter"] if "parameter" in task_input_dict else {}
        )

        inp = MultimodalEmbeddingInput()

        contents: List[dict] = []
        for embedding in data["embeddings"]:
            if embedding["type"] == "image-url":
                contents.append(
                    {
                        "type": "image",
                        "image": url_to_pil_image(embedding[embedding["type"]]),
                    }
                )
            elif embedding["type"] == "image-base64":
                contents.append(
                    {
                        "type": "image",
                        "image": base64_to_pil_image(embedding[embedding["type"]]),
                    }
                )
            elif embedding["type"] == "text":
                contents.append({"type": "text", "text": embedding[embedding["type"]]})
            else:
                raise InvalidInputException("non supported input type")

        inp.contents = contents

        # format
        if "format" in parameter:
            inp.format = str(parameter["format"])

        # dimensions
        if "dimensions" in parameter:
            inp.dimensions = int(parameter["dimensions"])

        # input-type
        if "input-type" in parameter:
            inp.input_type = str(parameter["input-type"])

        # truncate
        if "truncate" in parameter:
            inp.truncate = str(parameter["truncate"])

        input_list.append(inp)

    return input_list


def construct_task_embedding_output(
    request: Union[CallRequest, Request],
    indexes: List[List[int]],
    created_timestamps: List[List[int]],
    embeddings: List[List[list]],
) -> Union[CallResponse, List]:

    if not len(embeddings) == len(indexes) == len(created_timestamps):
        raise InvalidOutputShapeException
    if len(embeddings) > 0 and not (
        len(embeddings[0]) == len(indexes[0]) == len(created_timestamps[0])
    ):
        raise InvalidOutputShapeException

    task_outputs = []
    # data
    for index_list, created_timestamp_list, embedding_list in zip(
        indexes, created_timestamps, embeddings
    ):
        data = {}
        embeds = []
        for (
            index,
            created_timestamp,
            embed,
        ) in zip(index_list, created_timestamp_list, embedding_list):
            if isinstance(embed, np.ndarray):
                embed = embed.tolist()
            embeds.append(
                {
                    "index": index,
                    "vector": list(embed),
                    "created": created_timestamp,
                }
            )
        data["embeddings"] = embeds
        task_outputs.append({"data": data})

    if isinstance(request, Request):
        return task_outputs

    for i, o in enumerate(task_outputs):
        task_outputs[i] = dict_to_struct(o)

    return CallResponse(task_outputs=task_outputs)


async def parse_custom_input(
    request: Union[CallRequest, Request],
) -> List[dict]:

    # http test input
    if isinstance(request, Request):
        test_data: dict = await request.json()

        return [test_data]

    input_list = []
    for task_input in request.task_inputs:
        task_input_dict = json_format.MessageToDict(task_input)

        input_list.append(task_input_dict)

    return input_list


def construct_custom_output(
    request: Union[CallRequest, Request], outputs: List[dict]
) -> Union[CallResponse, List]:

    if isinstance(request, Request):
        return outputs

    for i, o in enumerate(outputs):
        outputs[i] = dict_to_struct(o)

    return CallResponse(task_outputs=outputs)



# pylint: disable=no-member,wrong-import-position,too-many-lines,no-name-in-module
from datetime import datetime
from typing import Callable, List, Optional

from google.protobuf import field_mask_pb2, timestamp_pb2
from google.protobuf.struct_pb2 import Struct

import instill.protogen.common.healthcheck.v1beta.healthcheck_pb2 as healthcheck
import instill.protogen.vdp.pipeline.v1beta.component_definition_pb2 as component_definition
import instill.protogen.vdp.pipeline.v1beta.integration_pb2 as integration_interface

# pipeline
import instill.protogen.vdp.pipeline.v1beta.pipeline_pb2 as pipeline_interface
import instill.protogen.vdp.pipeline.v1beta.pipeline_public_service_pb2_grpc as pipeline_service
import instill.protogen.vdp.pipeline.v1beta.secret_pb2 as secret_interface
from instill.clients.base import Client, RequestFactory
from instill.clients.instance import InstillInstance
from instill.helpers.const import HOST_URL_PROD
from instill.protogen.vdp.pipeline.v1beta import common_pb2
from instill.utils.error_handler import grpc_handler


class PipelineClient(Client):
    def __init__(
        self,
        api_token: str,
        lookup_func: Callable[[str], str],
        url: str = HOST_URL_PROD,
        secure: bool = True,
        requester_id: str = "",
        async_enabled: bool = False,
    ) -> None:
        self.host: InstillInstance = InstillInstance(
            pipeline_service.PipelinePublicServiceStub,
            url=url,
            token=api_token,
            secure=secure,
            async_enabled=async_enabled,
        )
        self.metadata = []

        if requester_id != "":
            requester_uid = lookup_func(requester_id)
            self.metadata = [("instill-requester-uid", requester_uid)]

    def close(self):
        if self.is_serving():
            self.host.channel.close()

    async def async_close(self):
        if self.is_serving():
            self.host.channel.close()

    @property
    def host(self):
        return self._host

    @host.setter
    def host(self, host: InstillInstance):
        self._host = host

    @property
    def metadata(self):
        return self._metadata

    @metadata.setter
    def metadata(self, metadata: List[tuple]):
        self._metadata = metadata

    def liveness(
        self, async_enabled: bool = False
    ) -> pipeline_interface.LivenessResponse:
        if async_enabled:
            return RequestFactory(
                method=self.host.async_client.Liveness,
                request=pipeline_interface.LivenessRequest(),
                metadata=self.host.metadata + self.metadata,
            ).send_async()

        return RequestFactory(
            method=self.host.client.Liveness,
            request=pipeline_interface.LivenessRequest(),
            metadata=self.host.metadata + self.metadata,
        ).send_sync()

    def readiness(
        self, async_enabled: bool = False
    ) -> pipeline_interface.ReadinessResponse:
        if async_enabled:
            return RequestFactory(
                method=self.host.async_client.Readiness,
                request=pipeline_interface.ReadinessRequest(),
                metadata=self.host.metadata + self.metadata,
            ).send_async()

        return RequestFactory(
            method=self.host.client.Readiness,
            request=pipeline_interface.ReadinessRequest(),
            metadata=self.host.metadata + self.metadata,
        ).send_sync()

    def is_serving(self) -> bool:
        try:
            return (
                self.readiness().health_check_response.status
                == healthcheck.HealthCheckResponse.SERVING_STATUS_SERVING
            )
        except Exception:
            return False

    @grpc_handler
    def get_hub_stats(
        self,
        async_enabled: bool = False,
    ) -> pipeline_interface.GetHubStatsResponse:
        if async_enabled:
            return RequestFactory(
                method=self.host.async_client.GetHubStats,
                request=pipeline_interface.GetHubStatsRequest(),
                metadata=self.host.metadata + self.metadata,
            ).send_async()

        return RequestFactory(
            method=self.host.client.GetHubStats,
            request=pipeline_interface.GetHubStatsRequest(),
            metadata=self.host.metadata + self.metadata,
        ).send_sync()

    @grpc_handler
    def lookup_pipeline(
        self,
        pipeline_uid: str,
        async_enabled: bool = False,
    ) -> pipeline_interface.LookUpPipelineResponse:
        if async_enabled:
            return RequestFactory(
                method=self.host.async_client.LookUpPipeline,
                request=pipeline_interface.LookUpPipelineRequest(
                    permalink=f"pipelines/{pipeline_uid}",
                    view=pipeline_interface.Pipeline.VIEW_RECIPE,
                ),
                metadata=self.host.metadata + self.metadata,
            ).send_async()

        return RequestFactory(
            method=self.host.client.LookUpPipeline,
            request=pipeline_interface.LookUpPipelineRequest(
                permalink=f"pipelines/{pipeline_uid}",
                view=pipeline_interface.Pipeline.VIEW_RECIPE,
            ),
            metadata=self.host.metadata + self.metadata,
        ).send_sync()

    @grpc_handler
    def list_public_pipelines(
        self,
        order_by: str = "",
        is_public: bool = True,
        filter_str: str = "",
        next_page_token: str = "",
        total_size: int = 100,
        show_deleted: bool = False,
        async_enabled: bool = False,
    ) -> pipeline_interface.ListPipelinesResponse:
        visibility = (
            pipeline_interface.Pipeline.Visibility.VISIBILITY_PUBLIC
            if is_public
            else pipeline_interface.Pipeline.Visibility.VISIBILITY_PRIVATE
        )

        if async_enabled:
            return RequestFactory(
                method=self.host.async_client.ListPipelines,
                request=pipeline_interface.ListPipelinesRequest(
                    filter=filter_str,
                    page_size=total_size,
                    page_token=next_page_token,
                    show_deleted=show_deleted,
                    view=pipeline_interface.Pipeline.VIEW_RECIPE,
                    visibility=visibility,
                    order_by=order_by,
                ),
                metadata=self.host.metadata + self.metadata,
            ).send_async()
        return RequestFactory(
            method=self.host.client.ListPipelines,
            request=pipeline_interface.ListPipelinesRequest(
                filter=filter_str,
                page_size=total_size,
                page_token=next_page_token,
                show_deleted=show_deleted,
                view=pipeline_interface.Pipeline.VIEW_RECIPE,
                visibility=visibility,
                order_by=order_by,
            ),
            metadata=self.host.metadata + self.metadata,
        ).send_sync()

    @grpc_handler
    def list_pipelines(
        self,
        namespace_id: str,
        order_by: str = "",
        is_public: bool = True,
        filter_str: str = "",
        next_page_token: str = "",
        total_size: int = 100,
        show_deleted: bool = False,
        async_enabled: bool = False,
    ) -> pipeline_interface.ListNamespacePipelinesResponse:
        visibility = (
            pipeline_interface.Pipeline.Visibility.VISIBILITY_PUBLIC
            if is_public
            else pipeline_interface.Pipeline.Visibility.VISIBILITY_PRIVATE
        )

        if async_enabled:
            return RequestFactory(
                method=self.host.async_client.ListNamespacePipelines,
                request=pipeline_interface.ListNamespacePipelinesRequest(
                    namespace_id=namespace_id,
                    filter=filter_str,
                    page_size=total_size,
                    page_token=next_page_token,
                    show_deleted=show_deleted,
                    view=pipeline_interface.Pipeline.VIEW_RECIPE,
                    visibility=visibility,
                    order_by=order_by,
                ),
                metadata=self.host.metadata + self.metadata,
            ).send_async()
        return RequestFactory(
            method=self.host.client.ListNamespacePipelines,
            request=pipeline_interface.ListNamespacePipelinesRequest(
                namespace_id=namespace_id,
                filter=filter_str,
                page_size=total_size,
                page_token=next_page_token,
                show_deleted=show_deleted,
                view=pipeline_interface.Pipeline.VIEW_RECIPE,
                visibility=visibility,
                order_by=order_by,
            ),
            metadata=self.host.metadata + self.metadata,
        ).send_sync()

    @grpc_handler
    def create_pipeline(
        self,
        namespace_id: str,
        pipeline_id: str,
        description: str,
        recipe: Optional[dict] = None,
        raw_recipe: str = "",
        async_enabled: bool = False,
    ) -> pipeline_interface.CreateNamespacePipelineResponse:
        pipeline = pipeline_interface.Pipeline(
            id=pipeline_id,
            description=description,
            raw_recipe=raw_recipe,
        )
        if recipe is None:
            recipe = {}
        pipeline.recipe.update(recipe)

        if async_enabled:
            return RequestFactory(
                method=self.host.async_client.CreateNamespacePipeline,
                request=pipeline_interface.CreateNamespacePipelineRequest(
                    namespace_id=namespace_id,
                    pipeline=pipeline,
                ),
                metadata=self.host.metadata + self.metadata,
            ).send_async()

        return RequestFactory(
            method=self.host.client.CreateNamespacePipeline,
            request=pipeline_interface.CreateNamespacePipelineRequest(
                namespace_id=namespace_id,
                pipeline=pipeline,
            ),
            metadata=self.host.metadata + self.metadata,
        ).send_sync()

    @grpc_handler
    def get_pipeline(
        self,
        namespace_id: str,
        pipeline_id: str,
        async_enabled: bool = False,
    ) -> pipeline_interface.GetNamespacePipelineResponse:
        if async_enabled:
            return RequestFactory(
                method=self.host.async_client.GetNamespacePipeline,
                request=pipeline_interface.GetNamespacePipelineRequest(
                    namespace_id=namespace_id,
                    pipeline_id=pipeline_id,
                    view=pipeline_interface.Pipeline.VIEW_RECIPE,
                ),
                metadata=self.host.metadata + self.metadata,
            ).send_async()

        return RequestFactory(
            method=self.host.client.GetNamespacePipeline,
            request=pipeline_interface.GetNamespacePipelineRequest(
                namespace_id=namespace_id,
                pipeline_id=pipeline_id,
                view=pipeline_interface.Pipeline.VIEW_RECIPE,
            ),
            metadata=self.host.metadata + self.metadata,
        ).send_sync()

    @grpc_handler
    def update_pipeline(
        self,
        namespace_id: str,
        pipeline_id: str,
        description: str,
        recipe: Optional[dict] = None,
        raw_recipe: str = "",
        async_enabled: bool = False,
    ) -> pipeline_interface.UpdateNamespacePipelineResponse:

        pipeline = pipeline_interface.Pipeline(
            name=f"namespaces/{namespace_id}/models/{pipeline_id}",
            id=pipeline_id,
            description=description,
            raw_recipe=raw_recipe,
        )

        if recipe is None:
            recipe = {}
        pipeline.recipe.update(recipe)

        update_mask = field_mask_pb2.FieldMask()
        update_mask.paths.extend(
            [
                "description",
                "recipe",
                "raw_recipe",
            ]
        )

        if async_enabled:
            return RequestFactory(
                method=self.host.async_client.UpdateNamespacePipeline,
                request=pipeline_interface.UpdateNamespacePipelineRequest(
                    namespace_id=namespace_id,
                    pipeline_id=pipeline_id,
                    pipeline=pipeline,
                    update_mask=update_mask,
                ),
                metadata=self.host.metadata + self.metadata,
            ).send_async()

        return RequestFactory(
            method=self.host.client.UpdateNamespacePipeline,
            request=pipeline_interface.UpdateNamespacePipelineRequest(
                namespace_id=namespace_id,
                pipeline_id=pipeline_id,
                pipeline=pipeline,
                update_mask=update_mask,
            ),
            metadata=self.host.metadata + self.metadata,
        ).send_sync()

    @grpc_handler
    def delete_pipeline(
        self,
        namespace_id: str,
        pipeline_id: str,
        async_enabled: bool = False,
    ) -> pipeline_interface.DeleteNamespacePipelineResponse:
        if async_enabled:
            return RequestFactory(
                method=self.host.async_client.DeleteNamespacePipeline,
                request=pipeline_interface.DeleteNamespacePipelineRequest(
                    namespace_id=namespace_id,
                    pipeline_id=pipeline_id,
                ),
                metadata=self.host.metadata + self.metadata,
            ).send_async()

        return RequestFactory(
            method=self.host.client.DeleteNamespacePipeline,
            request=pipeline_interface.DeleteNamespacePipelineRequest(
                namespace_id=namespace_id,
                pipeline_id=pipeline_id,
            ),
            metadata=self.host.metadata + self.metadata,
        ).send_sync()

    @grpc_handler
    def validate_pipeline(
        self,
        namespace_id: str,
        pipeline_id: str,
        async_enabled: bool = False,
    ) -> pipeline_interface.ValidateNamespacePipelineResponse:
        if async_enabled:
            return RequestFactory(
                method=self.host.async_client.ValidateNamespacePipeline,
                request=pipeline_interface.ValidateNamespacePipelineRequest(
                    namespace_id=namespace_id,
                    pipeline_id=pipeline_id,
                ),
                metadata=self.host.metadata + self.metadata,
            ).send_async()

        return RequestFactory(
            method=self.host.client.ValidateNamespacePipeline,
            request=pipeline_interface.ValidateNamespacePipelineRequest(
                namespace_id=namespace_id,
                pipeline_id=pipeline_id,
            ),
            metadata=self.host.metadata + self.metadata,
        ).send_sync()

    @grpc_handler
    def rename_pipeline(
        self,
        namespace_id: str,
        pipeline_id: str,
        new_pipeline_id: str,
        async_enabled: bool = False,
    ) -> pipeline_interface.RenameNamespacePipelineResponse:
        if async_enabled:
            return RequestFactory(
                method=self.host.async_client.RenameNamespacePipeline,
                request=pipeline_interface.RenameNamespacePipelineRequest(
                    namespace_id=namespace_id,
                    pipeline_id=pipeline_id,
                    new_pipeline_id=new_pipeline_id,
                ),
                metadata=self.host.metadata + self.metadata,
            ).send_async()

        return RequestFactory(
            method=self.host.client.RenameNamespacePipeline,
            request=pipeline_interface.RenameNamespacePipelineRequest(
                namespace_id=namespace_id,
                pipeline_id=pipeline_id,
                new_pipeline_id=new_pipeline_id,
            ),
            metadata=self.host.metadata + self.metadata,
        ).send_sync()

    @grpc_handler
    def clone_pipeline(
        self,
        namespace_id: str,
        pipeline_id: str,
        target: str,
        description: str,
        sharing_enabled: bool = False,
        sharing_role_executor: bool = True,
        async_enabled: bool = False,
    ) -> pipeline_interface.CloneNamespacePipelineResponse:
        sharing = common_pb2.Sharing()
        sharing.User.enabled = sharing_enabled
        if sharing_role_executor:
            sharing.User.role = common_pb2.Role.ROLE_EXECUTOR

        if async_enabled:
            return RequestFactory(
                method=self.host.async_client.CloneNamespacePipeline,
                request=pipeline_interface.CloneNamespacePipelineRequest(
                    namespace_id=namespace_id,
                    pipeline_id=pipeline_id,
                    description=description,
                    sharing=sharing,
                    target_namespace_id=namespace_id,
                    target_pipeline_id=target,
                ),
                metadata=self.host.metadata + self.metadata,
            ).send_async()

        return RequestFactory(
            method=self.host.client.CloneNamespacePipeline,
            request=pipeline_interface.CloneNamespacePipelineRequest(
                namespace_id=namespace_id,
                pipeline_id=pipeline_id,
                description=description,
                sharing=sharing,
                target_namespace_id=namespace_id,
                target_pipeline_id=target,
            ),
            metadata=self.host.metadata + self.metadata,
        ).send_sync()

    @grpc_handler
    def send_pipeline_event(
        self,
        namespace_id: str,
        pipeline_id: str,
        event: str,
        code: str,
        data: dict,
        async_enabled: bool = False,
    ) -> pipeline_interface.SendNamespacePipelineEventResponse:
        trigger_data = Struct()
        trigger_data.update(data)

        if async_enabled:
            return RequestFactory(
                method=self.host.async_client.SendNamespacePipelineEvent,
                request=pipeline_interface.SendNamespacePipelineEventRequest(
                    namespace_id=namespace_id,
                    pipeline_id=pipeline_id,
                    event=event,
                    code=code,
                    data=trigger_data,
                ),
                metadata=self.host.metadata + self.metadata,
            ).send_async()

        return RequestFactory(
            method=self.host.client.SendNamespacePipelineEvent,
            request=pipeline_interface.SendNamespacePipelineEventRequest(
                namespace_id=namespace_id,
                pipeline_id=pipeline_id,
                event=event,
                code=code,
                data=trigger_data,
            ),
            metadata=self.host.metadata + self.metadata,
        ).send_sync()

    @grpc_handler
    def send_pipeline_release_event(
        self,
        namespace_id: str,
        pipeline_id: str,
        release_id: str,
        event: str,
        code: str,
        data: Struct,
        async_enabled: bool = False,
    ) -> pipeline_interface.SendNamespacePipelineReleaseEventResponse:
        if async_enabled:
            return RequestFactory(
                method=self.host.async_client.SendNamespacePipelineReleaseEvent,
                request=pipeline_interface.SendNamespacePipelineReleaseEventRequest(
                    namespace_id=namespace_id,
                    pipeline_id=pipeline_id,
                    release_id=release_id,
                    event=event,
                    code=code,
                    data=data,
                ),
                metadata=self.host.metadata + self.metadata,
            ).send_async()

        return RequestFactory(
            method=self.host.client.SendNamespacePipelineReleaseEvent,
            request=pipeline_interface.SendNamespacePipelineReleaseEventRequest(
                namespace_id=namespace_id,
                pipeline_id=pipeline_id,
                release_id=release_id,
                event=event,
                code=code,
                data=data,
            ),
            metadata=self.host.metadata + self.metadata,
        ).send_sync()

    @grpc_handler
    def trigger(
        self,
        namespace_id: str,
        pipeline_id: str,
        data: List[dict],
        async_enabled: bool = False,
    ) -> pipeline_interface.TriggerNamespacePipelineResponse:
        request = pipeline_interface.TriggerNamespacePipelineRequest(
            namespace_id=namespace_id,
            pipeline_id=pipeline_id,
        )
        for d in data:
            trigger_data = pipeline_interface.TriggerData()
            trigger_data.variable.update(d)
            request.data.append(trigger_data)

        if async_enabled:
            return RequestFactory(
                method=self.host.async_client.TriggerNamespacePipeline,
                request=request,
                metadata=self.host.metadata + self.metadata,
            ).send_async()

        return RequestFactory(
            method=self.host.client.TriggerNamespacePipeline,
            request=request,
            metadata=self.host.metadata + self.metadata,
        ).send_sync()

    @grpc_handler
    def trigger_with_stream(
        self,
        namespace_id: str,
        pipeline_id: str,
        data: List[dict],
        async_enabled: bool = False,
    ) -> pipeline_interface.TriggerNamespacePipelineWithStreamResponse:
        request = pipeline_interface.TriggerNamespacePipelineWithStreamRequest(
            namespace_id=namespace_id,
            pipeline_id=pipeline_id,
        )
        for d in data:
            trigger_data = pipeline_interface.TriggerData()
            trigger_data.variable.update(d)
            request.data.append(trigger_data)

        if async_enabled:
            return RequestFactory(
                method=self.host.async_client.TriggerNamespacePipelineWithStream,
                request=request,
                metadata=self.host.metadata + self.metadata,
            ).send_async()

        return RequestFactory(
            method=self.host.client.TriggerNamespacePipelineWithStream,
            request=request,
            metadata=self.host.metadata + self.metadata,
        ).send_sync()

    @grpc_handler
    def trigger_async(
        self,
        namespace_id: str,
        pipeline_id: str,
        data: List[dict],
        async_enabled: bool = False,
    ) -> pipeline_interface.TriggerAsyncNamespacePipelineResponse:
        request = pipeline_interface.TriggerAsyncNamespacePipelineRequest(
            namespace_id=namespace_id,
            pipeline_id=pipeline_id,
        )
        for d in data:
            trigger_data = pipeline_interface.TriggerData()
            trigger_data.variable.update(d)
            request.data.append(trigger_data)

        if async_enabled:
            return RequestFactory(
                method=self.host.async_client.TriggerAsyncNamespacePipeline,
                request=request,
                metadata=self.host.metadata + self.metadata,
            ).send_async()

        return RequestFactory(
            method=self.host.client.TriggerAsyncNamespacePipeline,
            request=request,
            metadata=self.host.metadata + self.metadata,
        ).send_sync()

    @grpc_handler
    def create_pipeline_release(
        self,
        namespace_id: str,
        pipeline_id: str,
        release_id: str,
        name: str = "",
        description: str = "",
        alias: str = "",
        recipe: Optional[dict] = None,
        raw_recipe: str = "",
        metadata: Optional[dict] = None,
        async_enabled: bool = False,
    ) -> pipeline_interface.CreateNamespacePipelineReleaseResponse:
        release = pipeline_interface.PipelineRelease(
            id=release_id,
            name=name,
            description=description,
            alias=alias,
            raw_recipe=raw_recipe,
        )

        if recipe is None:
            recipe = {}
        release.recipe.update(recipe)
        if metadata is None:
            metadata = {}
        release.metadata.update(metadata)

        if async_enabled:
            return RequestFactory(
                method=self.host.async_client.CreateNamespacePipelineRelease,
                request=pipeline_interface.CreateNamespacePipelineReleaseRequest(
                    namespace_id=namespace_id,
                    pipeline_id=pipeline_id,
                    release=release,
                ),
                metadata=self.host.metadata + self.metadata,
            ).send_async()

        return RequestFactory(
            method=self.host.client.CreateNamespacePipelineRelease,
            request=pipeline_interface.CreateNamespacePipelineReleaseRequest(
                namespace_id=namespace_id,
                pipeline_id=pipeline_id,
                release=release,
            ),
            metadata=self.host.metadata + self.metadata,
        ).send_sync()

    @grpc_handler
    def list_pipeline_releases(
        self,
        namespace_id: str,
        pipeline_id: str,
        total_size: int = 10,
        next_page_token: str = "",
        filter_str: str = "",
        show_deleted: bool = False,
        async_enabled: bool = False,
    ) -> pipeline_interface.ListNamespacePipelineReleasesResponse:
        if async_enabled:
            return RequestFactory(
                method=self.host.async_client.ListNamespacePipelineReleases,
                request=pipeline_interface.ListNamespacePipelineReleasesRequest(
                    namespace_id=namespace_id,
                    pipeline_id=pipeline_id,
                    view=pipeline_interface.Pipeline.VIEW_RECIPE,
                    page_size=total_size,
                    page_token=next_page_token,
                    filter=filter_str,
                    show_deleted=show_deleted,
                ),
                metadata=self.host.metadata + self.metadata,
            ).send_async()

        return RequestFactory(
            method=self.host.client.ListNamespacePipelineReleases,
            request=pipeline_interface.ListNamespacePipelineReleasesRequest(
                namespace_id=namespace_id,
                pipeline_id=pipeline_id,
                view=pipeline_interface.Pipeline.VIEW_RECIPE,
                page_size=total_size,
                page_token=next_page_token,
                filter=filter_str,
                show_deleted=show_deleted,
            ),
            metadata=self.host.metadata + self.metadata,
        ).send_sync()

    @grpc_handler
    def get_pipeline_release(
        self,
        namespace_id: str,
        pipeline_id: str,
        release_id: str,
        async_enabled: bool = False,
    ) -> pipeline_interface.GetNamespacePipelineReleaseResponse:
        if async_enabled:
            return RequestFactory(
                method=self.host.async_client.GetNamespacePipelineRelease,
                request=pipeline_interface.GetNamespacePipelineReleaseRequest(
                    namespace_id=namespace_id,
                    pipeline_id=pipeline_id,
                    release_id=release_id,
                    view=pipeline_interface.Pipeline.VIEW_RECIPE,
                ),
                metadata=self.host.metadata + self.metadata,
            ).send_async()

        return RequestFactory(
            method=self.host.client.GetNamespacePipelineRelease,
            request=pipeline_interface.GetNamespacePipelineReleaseRequest(
                namespace_id=namespace_id,
                pipeline_id=pipeline_id,
                release_id=release_id,
                view=pipeline_interface.Pipeline.VIEW_RECIPE,
            ),
            metadata=self.host.metadata + self.metadata,
        ).send_sync()

    @grpc_handler
    def update_pipeline_release(
        self,
        namespace_id: str,
        pipeline_id: str,
        release_id: str,
        description: str = "",
        recipe: Optional[dict] = None,
        raw_recipe: str = "",
        async_enabled: bool = False,
    ) -> pipeline_interface.UpdateNamespacePipelineReleaseResponse:
        release = pipeline_interface.PipelineRelease(
            name=f"namespaces/{namespace_id}/models/{pipeline_id}/releases/{release_id}",
            id=release_id,
            description=description,
            raw_recipe=raw_recipe,
        )

        if recipe is None:
            recipe = {}
        release.recipe.update(recipe)

        update_mask = field_mask_pb2.FieldMask()
        update_mask.paths.extend(
            [
                "recipe",
                "description",
            ]
        )

        if async_enabled:
            return RequestFactory(
                method=self.host.async_client.UpdateNamespacePipelineRelease,
                request=pipeline_interface.UpdateNamespacePipelineReleaseRequest(
                    namespace_id=namespace_id,
                    pipeline_id=pipeline_id,
                    release_id=release_id,
                    release=release,
                    update_mask=update_mask,
                ),
                metadata=self.host.metadata + self.metadata,
            ).send_async()

        return RequestFactory(
            method=self.host.client.UpdateNamespacePipelineRelease,
            request=pipeline_interface.UpdateNamespacePipelineReleaseRequest(
                namespace_id=namespace_id,
                pipeline_id=pipeline_id,
                release_id=release_id,
                release=release,
                update_mask=update_mask,
            ),
            metadata=self.host.metadata + self.metadata,
        ).send_sync()

    @grpc_handler
    def delete_pipeline_release(
        self,
        namespace_id: str,
        pipeline_id: str,
        release_id: str,
        async_enabled: bool = False,
    ) -> pipeline_interface.DeleteNamespacePipelineReleaseResponse:
        if async_enabled:
            return RequestFactory(
                method=self.host.async_client.DeleteNamespacePipelineRelease,
                request=pipeline_interface.DeleteNamespacePipelineReleaseRequest(
                    namespace_id=namespace_id,
                    pipeline_id=pipeline_id,
                    release_id=release_id,
                ),
                metadata=self.host.metadata + self.metadata,
            ).send_async()

        return RequestFactory(
            method=self.host.client.DeleteNamespacePipelineRelease,
            request=pipeline_interface.DeleteNamespacePipelineReleaseRequest(
                namespace_id=namespace_id,
                pipeline_id=pipeline_id,
                release_id=release_id,
            ),
            metadata=self.host.metadata + self.metadata,
        ).send_sync()

    @grpc_handler
    def clone_pipeline_release(
        self,
        namespace_id: str,
        pipeline_id: str,
        release_id: str,
        target: str,
        description: str,
        sharing_enabled: bool = False,
        sharing_role_executor: bool = True,
        async_enabled: bool = False,
    ) -> pipeline_interface.CloneNamespacePipelineReleaseResponse:
        sharing = common_pb2.Sharing()
        sharing.User.enabled = sharing_enabled
        if sharing_role_executor:
            sharing.User.role = common_pb2.Role.ROLE_EXECUTOR

        if async_enabled:
            return RequestFactory(
                method=self.host.async_client.CloneNamespacePipelineRelease,
                request=pipeline_interface.CloneNamespacePipelineReleaseRequest(
                    namespace_id=namespace_id,
                    pipeline_id=pipeline_id,
                    release_id=release_id,
                    description=description,
                    sharing=sharing,
                    target_namespace_id=namespace_id,
                    target_pipeline_id=target,
                ),
                metadata=self.host.metadata + self.metadata,
            ).send_async()

        return RequestFactory(
            method=self.host.client.CloneNamespacePipelineRelease,
            request=pipeline_interface.CloneNamespacePipelineReleaseRequest(
                namespace_id=namespace_id,
                pipeline_id=pipeline_id,
                release_id=release_id,
                description=description,
                sharing=sharing,
                target_namespace_id=namespace_id,
                target_pipeline_id=target,
            ),
            metadata=self.host.metadata + self.metadata,
        ).send_sync()

    @grpc_handler
    def trigger_release(
        self,
        namespace_id: str,
        pipeline_id: str,
        release_id: str,
        data: List[dict],
        async_enabled: bool = False,
    ) -> pipeline_interface.TriggerNamespacePipelineReleaseResponse:
        request = pipeline_interface.TriggerNamespacePipelineReleaseRequest(
            namespace_id=namespace_id,
            pipeline_id=pipeline_id,
            release_id=release_id,
        )
        for d in data:
            trigger_data = pipeline_interface.TriggerData()
            trigger_data.variable.update(d)
            request.data.append(trigger_data)

        if async_enabled:
            return RequestFactory(
                method=self.host.async_client.TriggerNamespacePipelineRelease,
                request=request,
                metadata=self.host.metadata + self.metadata,
            ).send_async()

        return RequestFactory(
            method=self.host.client.TriggerNamespacePipelineRelease,
            request=request,
            metadata=self.host.metadata + self.metadata,
        ).send_sync()

    @grpc_handler
    def trigger_async_release(
        self,
        namespace_id: str,
        pipeline_id: str,
        release_id: str,
        data: List[dict],
        async_enabled: bool = False,
    ) -> pipeline_interface.TriggerAsyncNamespacePipelineReleaseResponse:
        request = pipeline_interface.TriggerAsyncNamespacePipelineReleaseRequest(
            namespace_id=namespace_id,
            pipeline_id=pipeline_id,
            release_id=release_id,
        )
        for d in data:
            trigger_data = pipeline_interface.TriggerData()
            trigger_data.variable.update(d)
            request.data.append(trigger_data)

        if async_enabled:
            return RequestFactory(
                method=self.host.async_client.TriggerAsyncNamespacePipelineRelease,
                request=request,
                metadata=self.host.metadata + self.metadata,
            ).send_async()

        return RequestFactory(
            method=self.host.client.TriggerAsyncNamespacePipelineRelease,
            request=request,
            metadata=self.host.metadata + self.metadata,
        ).send_sync()

    @grpc_handler
    def create_secret(
        self,
        namespace_id: str,
        secret_id: str,
        description: str,
        secret_name: str = "",
        value: str = "",
        async_enabled: bool = False,
    ) -> secret_interface.CreateNamespaceSecretResponse:
        secret = secret_interface.Secret(
            id=secret_id, name=secret_name, description=description, value=value
        )

        if async_enabled:
            return RequestFactory(
                method=self.host.async_client.CreateNamespaceSecret,
                request=secret_interface.CreateNamespaceSecretRequest(
                    namespace_id=namespace_id,
                    secret=secret,
                ),
                metadata=self.host.metadata + self.metadata,
            ).send_async()

        return RequestFactory(
            method=self.host.client.CreateNamespaceSecret,
            request=secret_interface.CreateNamespaceSecretRequest(
                namespace_id=namespace_id,
                secret=secret,
            ),
            metadata=self.host.metadata + self.metadata,
        ).send_sync()

    @grpc_handler
    def list_secrets(
        self,
        namespace_id: str,
        total_size: int = 10,
        next_page_token: str = "",
        async_enabled: bool = False,
    ) -> secret_interface.ListNamespaceSecretsResponse:
        if async_enabled:
            return RequestFactory(
                method=self.host.async_client.ListNamespaceSecrets,
                request=secret_interface.ListNamespaceSecretsRequest(
                    namespace_id=namespace_id,
                    page_size=total_size,
                    page_token=next_page_token,
                ),
                metadata=self.host.metadata + self.metadata,
            ).send_async()

        return RequestFactory(
            method=self.host.client.ListNamespaceSecrets,
            request=secret_interface.ListNamespaceSecretsRequest(
                namespace_id=namespace_id,
                page_size=total_size,
                page_token=next_page_token,
            ),
            metadata=self.host.metadata + self.metadata,
        ).send_sync()

    @grpc_handler
    def get_secret(
        self,
        namespace_id: str,
        secret_id: str,
        async_enabled: bool = False,
    ) -> secret_interface.GetNamespaceSecretResponse:
        if async_enabled:
            return RequestFactory(
                method=self.host.async_client.GetNamespaceSecret,
                request=secret_interface.GetNamespaceSecretRequest(
                    namespace_id=namespace_id,
                    secret_id=secret_id,
                ),
                metadata=self.host.metadata + self.metadata,
            ).send_async()

        return RequestFactory(
            method=self.host.client.GetNamespaceSecret,
            request=secret_interface.GetNamespaceSecretRequest(
                namespace_id=namespace_id,
                secret_id=secret_id,
            ),
            metadata=self.host.metadata + self.metadata,
        ).send_sync()

    @grpc_handler
    def update_secret(
        self,
        namespace_id: str,
        secret_id: str,
        description: str,
        secret_name: str = "",
        value: str = "",
        async_enabled: bool = False,
    ) -> secret_interface.UpdateNamespaceSecretResponse:
        secret = secret_interface.Secret(
            name=secret_name, description=description, value=value
        )

        if async_enabled:
            return RequestFactory(
                method=self.host.async_client.UpdateNamespaceSecret,
                request=secret_interface.UpdateNamespaceSecretRequest(
                    namespace_id=namespace_id,
                    secret_id=secret_id,
                    secret=secret,
                ),
                metadata=self.host.metadata + self.metadata,
            ).send_async()

        return RequestFactory(
            method=self.host.client.UpdateNamespaceSecret,
            request=secret_interface.UpdateNamespaceSecretRequest(
                namespace_id=namespace_id,
                secret_id=secret_id,
                secret=secret,
            ),
            metadata=self.host.metadata + self.metadata,
        ).send_sync()

    @grpc_handler
    def delete_secret(
        self,
        namespace_id: str,
        secret_id: str,
        async_enabled: bool = False,
    ) -> secret_interface.DeleteNamespaceSecretResponse:
        if async_enabled:
            return RequestFactory(
                method=self.host.async_client.DeleteNamespaceSecret,
                request=secret_interface.DeleteNamespaceSecretRequest(
                    namespace_id=namespace_id,
                    secret_id=secret_id,
                ),
                metadata=self.host.metadata + self.metadata,
            ).send_async()

        return RequestFactory(
            method=self.host.client.DeleteNamespaceSecret,
            request=secret_interface.DeleteNamespaceSecretRequest(
                namespace_id=namespace_id,
                secret_id=secret_id,
            ),
            metadata=self.host.metadata + self.metadata,
        ).send_sync()

    @grpc_handler
    def list_component_definitions(
        self,
        page: int,
        total_size: int = 100,
        filter_str: str = "",
        async_enabled: bool = False,
    ) -> component_definition.ListComponentDefinitionsResponse:
        if async_enabled:
            return RequestFactory(
                method=self.host.async_client.ListComponentDefinitions,
                request=component_definition.ListComponentDefinitionsRequest(
                    view=component_definition.ComponentDefinition.VIEW_FULL,
                    page=page,
                    page_size=total_size,
                    filter=filter_str,
                ),
                metadata=self.host.metadata + self.metadata,
            ).send_async()

        return RequestFactory(
            method=self.host.client.ListComponentDefinitions,
            request=component_definition.ListComponentDefinitionsRequest(
                view=component_definition.ComponentDefinition.VIEW_FULL,
                page=page,
                page_size=total_size,
                filter=filter_str,
            ),
            metadata=self.host.metadata + self.metadata,
        ).send_sync()

    @grpc_handler
    def get_operation(
        self,
        operation_id: str,
        async_enabled: bool = False,
    ) -> pipeline_interface.GetOperationResponse:
        if async_enabled:
            return RequestFactory(
                method=self.host.async_client.GetOperation,
                request=pipeline_interface.GetOperationRequest(
                    operation_id=operation_id,
                ),
                metadata=self.host.metadata + self.metadata,
            ).send_async()

        return RequestFactory(
            method=self.host.client.GetOperation,
            request=pipeline_interface.GetOperationRequest(
                operation_id=operation_id,
            ),
            metadata=self.host.metadata + self.metadata,
        ).send_sync()

    @grpc_handler
    def list_connector_definitions(
        self,
        total_size: int = 100,
        next_page_token: str = "",
        filter_str: str = "",
        async_enabled: bool = False,
    ) -> component_definition.ListConnectorDefinitionsResponse:
        if async_enabled:
            return RequestFactory(
                method=self.host.async_client.ListConnectorDefinitions,
                request=component_definition.ListConnectorDefinitionsRequest(
                    view=component_definition.ComponentDefinition.VIEW_FULL,
                    page_size=total_size,
                    page_token=next_page_token,
                    filter=filter_str,
                ),
                metadata=self.host.metadata + self.metadata,
            ).send_async()

        return RequestFactory(
            method=self.host.client.ListConnectorDefinitions,
            request=component_definition.ListConnectorDefinitionsRequest(
                view=component_definition.ComponentDefinition.VIEW_FULL,
                page_size=total_size,
                page_token=next_page_token,
                filter=filter_str,
            ),
            metadata=self.host.metadata + self.metadata,
        ).send_sync()

    @grpc_handler
    def get_connector_definition(
        self,
        namespace_id: str,
        name: str,
        async_enabled: bool = False,
    ) -> component_definition.GetConnectorDefinitionResponse:
        if async_enabled:
            return RequestFactory(
                method=self.host.async_client.GetConnectorDefinition,
                request=component_definition.GetConnectorDefinitionRequest(
                    name=f"namespaces/{namespace_id}/pipelines/{name}",
                    view=component_definition.ComponentDefinition.VIEW_FULL,
                ),
                metadata=self.host.metadata + self.metadata,
            ).send_async()

        return RequestFactory(
            method=self.host.client.GetConnectorDefinition,
            request=component_definition.GetConnectorDefinitionRequest(
                name=f"namespaces/{namespace_id}/pipelines/{name}",
                view=component_definition.ComponentDefinition.VIEW_FULL,
            ),
            metadata=self.host.metadata + self.metadata,
        ).send_sync()

    @grpc_handler
    def list_operator_definitions(
        self,
        total_size: int = 100,
        next_page_token: str = "",
        filter_str: str = "",
        async_enabled: bool = False,
    ) -> component_definition.ListOperatorDefinitionsResponse:
        if async_enabled:
            return RequestFactory(
                method=self.host.async_client.ListOperatorDefinitions,
                request=component_definition.ListOperatorDefinitionsRequest(
                    view=component_definition.ComponentDefinition.VIEW_FULL,
                    page_size=total_size,
                    page_token=next_page_token,
                    filter=filter_str,
                ),
                metadata=self.host.metadata + self.metadata,
            ).send_async()

        return RequestFactory(
            method=self.host.client.ListOperatorDefinitions,
            request=component_definition.ListOperatorDefinitionsRequest(
                view=component_definition.ComponentDefinition.VIEW_FULL,
                page_size=total_size,
                page_token=next_page_token,
                filter=filter_str,
            ),
            metadata=self.host.metadata + self.metadata,
        ).send_sync()

    @grpc_handler
    def get_operator_definition(
        self,
        namespace_id: str,
        name: str,
        async_enabled: bool = False,
    ) -> component_definition.GetOperatorDefinitionResponse:
        if async_enabled:
            return RequestFactory(
                method=self.host.async_client.GetOperatorDefinition,
                request=component_definition.GetOperatorDefinitionRequest(
                    name=f"namespaces/{namespace_id}/pipelines/{name}",
                    view=component_definition.ComponentDefinition.VIEW_FULL,
                ),
                metadata=self.host.metadata + self.metadata,
            ).send_async()

        return RequestFactory(
            method=self.host.client.GetOperatorDefinition,
            request=component_definition.GetOperatorDefinitionRequest(
                name=f"namespaces/{namespace_id}/pipelines/{name}",
                view=component_definition.ComponentDefinition.VIEW_FULL,
            ),
            metadata=self.host.metadata + self.metadata,
        ).send_sync()

    @grpc_handler
    def check_name(
        self,
        namespace_id: str,
        name: str,
        async_enabled: bool = False,
    ) -> common_pb2.CheckNameResponse:
        if async_enabled:
            return RequestFactory(
                method=self.host.async_client.CheckName,
                request=common_pb2.CheckNameRequest(
                    name=f"namespaces/{namespace_id}/pipelines/{name}",
                ),
                metadata=self.host.metadata + self.metadata,
            ).send_async()

        return RequestFactory(
            method=self.host.client.CheckName,
            request=common_pb2.CheckNameRequest(
                name=f"namespaces/{namespace_id}/pipelines/{name}",
            ),
            metadata=self.host.metadata + self.metadata,
        ).send_sync()

    @grpc_handler
    def list_pipeline_runs(
        self,
        namespace_id: str,
        pipeline_id: str,
        page: int = 0,
        total_size: int = 10,
        filter_str: str = "",
        order_by: str = "",
        async_enabled: bool = False,
    ) -> pipeline_interface.ListPipelineRunsResponse:
        if async_enabled:
            return RequestFactory(
                method=self.host.async_client.ListPipelineRuns,
                request=pipeline_interface.ListPipelineRunsRequest(
                    namespace_id=namespace_id,
                    pipeline_id=pipeline_id,
                    page=page,
                    page_size=total_size,
                    filter=filter_str,
                    order_by=order_by,
                ),
                metadata=self.host.metadata + self.metadata,
            ).send_async()

        return RequestFactory(
            method=self.host.client.ListPipelineRuns,
            request=pipeline_interface.ListPipelineRunsRequest(
                namespace_id=namespace_id,
                pipeline_id=pipeline_id,
                page=page,
                page_size=total_size,
                filter=filter_str,
                order_by=order_by,
            ),
            metadata=self.host.metadata + self.metadata,
        ).send_sync()

    @grpc_handler
    def list_component_runs(
        self,
        pipeline_run_id: str,
        page: int = 0,
        total_size: int = 10,
        filter_str: str = "",
        order_by: str = "",
        async_enabled: bool = False,
    ) -> pipeline_interface.ListComponentRunsResponse:
        if async_enabled:
            return RequestFactory(
                method=self.host.async_client.ListComponentRuns,
                request=pipeline_interface.ListComponentRunsRequest(
                    pipeline_run_id=pipeline_run_id,
                    view=pipeline_interface.Pipeline.VIEW_RECIPE,
                    page=page,
                    page_size=total_size,
                    filter=filter_str,
                    order_by=order_by,
                ),
                metadata=self.host.metadata + self.metadata,
            ).send_async()

        return RequestFactory(
            method=self.host.client.ListComponentRuns,
            request=pipeline_interface.ListComponentRunsRequest(
                pipeline_run_id=pipeline_run_id,
                view=pipeline_interface.Pipeline.VIEW_RECIPE,
                page=page,
                page_size=total_size,
                filter=filter_str,
                order_by=order_by,
            ),
            metadata=self.host.metadata + self.metadata,
        ).send_sync()

    @grpc_handler
    def list_pipeline_runs_by_credit_owner(
        self,
        start: datetime,
        stop: datetime,
        page: int = 0,
        total_size: int = 10,
        filter_str: str = "",
        order_by: str = "",
        async_enabled: bool = False,
    ) -> pipeline_interface.ListPipelineRunsByCreditOwnerResponse:
        start_timestamp = timestamp_pb2.Timestamp()
        start_timestamp.FromDatetime(start)
        stop_timestamp = timestamp_pb2.Timestamp()
        stop_timestamp.FromDatetime(stop)

        if async_enabled:
            return RequestFactory(
                method=self.host.async_client.ListPipelineRunsByCreditOwner,
                request=pipeline_interface.ListPipelineRunsByCreditOwnerRequest(
                    start=start_timestamp,
                    stop=stop_timestamp,
                    page=page,
                    page_size=total_size,
                    filter=filter_str,
                    order_by=order_by,
                ),
                metadata=self.host.metadata + self.metadata,
            ).send_async()

        return RequestFactory(
            method=self.host.client.ListPipelineRunsByCreditOwner,
            request=pipeline_interface.ListPipelineRunsByCreditOwnerRequest(
                start=start_timestamp,
                stop=stop_timestamp,
                page=page,
                page_size=total_size,
                filter=filter_str,
                order_by=order_by,
            ),
            metadata=self.host.metadata + self.metadata,
        ).send_sync()

    @grpc_handler
    def list_connections(
        self,
        namespace_id: str,
        total_size: int = 10,
        next_page_token: str = "",
        filter_str: str = "",
        async_enabled: bool = False,
    ) -> integration_interface.ListNamespaceConnectionsResponse:
        if async_enabled:
            return RequestFactory(
                method=self.host.async_client.ListNamespaceConnections,
                request=integration_interface.ListNamespaceConnectionsRequest(
                    namespace_id=namespace_id,
                    page_size=total_size,
                    page_token=next_page_token,
                    filter=filter_str,
                ),
                metadata=self.host.metadata + self.metadata,
            ).send_async()

        return RequestFactory(
            method=self.host.client.ListNamespaceConnections,
            request=integration_interface.ListNamespaceConnectionsRequest(
                namespace_id=namespace_id,
                page_size=total_size,
                page_token=next_page_token,
                filter=filter_str,
            ),
            metadata=self.host.metadata + self.metadata,
        ).send_sync()

    @grpc_handler
    def get_connection(
        self,
        namespace_id: str,
        connection_id: str,
        async_enabled: bool = False,
    ) -> integration_interface.GetNamespaceConnectionResponse:
        if async_enabled:
            return RequestFactory(
                method=self.host.async_client.GetNamespaceConnection,
                request=integration_interface.GetNamespaceConnectionRequest(
                    namespace_id=namespace_id,
                    connection_id=connection_id,
                    view=pipeline_interface.Pipeline.VIEW_RECIPE,
                ),
                metadata=self.host.metadata + self.metadata,
            ).send_async()

        return RequestFactory(
            method=self.host.client.GetNamespaceConnection,
            request=integration_interface.GetNamespaceConnectionRequest(
                namespace_id=namespace_id,
                connection_id=connection_id,
                view=pipeline_interface.Pipeline.VIEW_RECIPE,
            ),
            metadata=self.host.metadata + self.metadata,
        ).send_sync()

    @grpc_handler
    def create_connection(
        self,
        namespace_id: str,
        integration_id: str,
        connection_id: str,
        scopes: Optional[List[str]] = None,
        identity: Optional[str] = None,
        setup: Optional[dict] = None,
        o_auth_access_details: Optional[dict] = None,
        is_oauth: bool = False,
        async_enabled: bool = False,
    ) -> integration_interface.CreateNamespaceConnectionResponse:
        if scopes is None:
            scopes = []
        connection = integration_interface.Connection(
            namespace_id=namespace_id,
            integration_id=integration_id,
            id=connection_id,
            identity=identity,
            scopes=scopes,
        )

        setup = {} if setup is None else setup
        connection.setup.Clear()
        connection.setup.update(setup)
        if o_auth_access_details is None:
            o_auth_access_details = {}
        connection.o_auth_access_details.update(o_auth_access_details)

        if is_oauth:
            connection.method = integration_interface.Connection.Method.METHOD_OAUTH
        else:
            connection.method = (
                integration_interface.Connection.Method.METHOD_DICTIONARY
            )

        if async_enabled:
            return RequestFactory(
                method=self.host.async_client.CreateNamespaceConnection,
                request=integration_interface.CreateNamespaceConnectionRequest(
                    connection=connection,
                ),
                metadata=self.host.metadata + self.metadata,
            ).send_async()

        return RequestFactory(
            method=self.host.client.CreateNamespaceConnection,
            request=integration_interface.CreateNamespaceConnectionRequest(
                connection=connection,
            ),
            metadata=self.host.metadata + self.metadata,
        ).send_sync()

    @grpc_handler
    def update_connection(
        self,
        namespace_id: str,
        integration_id: str,
        connection_id: str,
        scopes: Optional[List[str]] = None,
        identity: Optional[str] = None,
        setup: Optional[dict] = None,
        # o_auth_access_details: Optional[dict] = None,
        is_oauth: bool = False,
        async_enabled: bool = False,
    ) -> integration_interface.UpdateNamespaceConnectionResponse:
        if scopes is None:
            scopes = []
        connection = integration_interface.Connection(
            namespace_id=namespace_id,
            integration_id=integration_id,
            scopes=scopes,
            identity=identity,
        )

        if setup is None:
            setup = {}
        connection.setup.Clear()
        connection.setup.update(setup)
        # if o_auth_access_details is None:
        #     o_auth_access_details = {}
        # connection.o_auth_access_details.Clear()
        # connection.o_auth_access_details.update(o_auth_access_details)

        update_mask = field_mask_pb2.FieldMask()
        update_mask.paths.extend(
            [
                "setup",
                "scopes",
                "identity",
            ]
        )

        if is_oauth:
            connection.method = integration_interface.Connection.Method.METHOD_OAUTH
        else:
            connection.method = (
                integration_interface.Connection.Method.METHOD_DICTIONARY
            )

        if async_enabled:
            return RequestFactory(
                method=self.host.async_client.UpdateNamespaceConnection,
                request=integration_interface.UpdateNamespaceConnectionRequest(
                    connection_id=connection_id,
                    connection=connection,
                    update_mask=update_mask,
                ),
                metadata=self.host.metadata + self.metadata,
            ).send_async()

        return RequestFactory(
            method=self.host.client.UpdateNamespaceConnection,
            request=integration_interface.UpdateNamespaceConnectionRequest(
                connection_id=connection_id,
                connection=connection,
                update_mask=update_mask,
            ),
            metadata=self.host.metadata + self.metadata,
        ).send_sync()

    @grpc_handler
    def delete_connection(
        self,
        namespace_id: str,
        connection_id: str,
        async_enabled: bool = False,
    ) -> integration_interface.DeleteNamespaceConnectionResponse:
        if async_enabled:
            return RequestFactory(
                method=self.host.async_client.DeleteNamespaceConnection,
                request=integration_interface.DeleteNamespaceConnectionRequest(
                    namespace_id=namespace_id,
                    connection_id=connection_id,
                ),
                metadata=self.host.metadata + self.metadata,
            ).send_async()

        return RequestFactory(
            method=self.host.client.DeleteNamespaceConnection,
            request=integration_interface.DeleteNamespaceConnectionRequest(
                namespace_id=namespace_id,
                connection_id=connection_id,
            ),
            metadata=self.host.metadata + self.metadata,
        ).send_sync()

    @grpc_handler
    def test_connection(
        self,
        namespace_id: str,
        connection_id: str,
        async_enabled: bool = False,
    ) -> integration_interface.TestNamespaceConnectionResponse:
        if async_enabled:
            return RequestFactory(
                method=self.host.async_client.TestNamespaceConnection,
                request=integration_interface.TestNamespaceConnectionRequest(
                    namespace_id=namespace_id,
                    connection_id=connection_id,
                ),
                metadata=self.host.metadata + self.metadata,
            ).send_async()

        return RequestFactory(
            method=self.host.client.TestNamespaceConnection,
            request=integration_interface.TestNamespaceConnectionRequest(
                namespace_id=namespace_id,
                connection_id=connection_id,
            ),
            metadata=self.host.metadata + self.metadata,
        ).send_sync()

    @grpc_handler
    def list_integrations(
        self,
        total_size: int = 10,
        next_page_token: str = "",
        filter_str: str = "",
        async_enabled: bool = False,
    ) -> integration_interface.ListIntegrationsResponse:
        if async_enabled:
            return RequestFactory(
                method=self.host.async_client.ListIntegrations,
                request=integration_interface.ListIntegrationsRequest(
                    page_size=total_size,
                    page_token=next_page_token,
                    filter=filter_str,
                ),
                metadata=self.host.metadata + self.metadata,
            ).send_async()

        return RequestFactory(
            method=self.host.client.ListIntegrations,
            request=integration_interface.ListIntegrationsRequest(
                page_size=total_size,
                page_token=next_page_token,
                filter=filter_str,
            ),
            metadata=self.host.metadata + self.metadata,
        ).send_sync()

    @grpc_handler
    def get_integration(
        self,
        integration_id: str,
        async_enabled: bool = False,
    ) -> integration_interface.GetIntegrationResponse:
        if async_enabled:
            return RequestFactory(
                method=self.host.async_client.GetIntegration,
                request=integration_interface.GetIntegrationRequest(
                    integration_id=integration_id,
                    view=pipeline_interface.Pipeline.VIEW_RECIPE,
                ),
                metadata=self.host.metadata + self.metadata,
            ).send_async()

        return RequestFactory(
            method=self.host.client.GetIntegration,
            request=integration_interface.GetIntegrationRequest(
                integration_id=integration_id,
                view=pipeline_interface.Pipeline.VIEW_RECIPE,
            ),
            metadata=self.host.metadata + self.metadata,
        ).send_sync()



# pylint: disable=redefined-outer-name,unused-variable,expression-not-assigned,no-name-in-module

import instill.protogen.core.mgmt.v1beta.mgmt_public_service_pb2_grpc as mgmt_service
import instill.protogen.model.model.v1alpha.model_public_service_pb2_grpc as model_service
import instill.protogen.vdp.pipeline.v1beta.pipeline_public_service_pb2_grpc as pipeline_service
from instill.clients import MgmtClient, ModelClient, PipelineClient
from instill.clients.instance import InstillInstance


def mock(_: str):
    return ""


def describe_client():
    def describe_host():
        def when_not_set(expect):
            mgmt_client = MgmtClient("")
            expect(mgmt_client.host) is not None
            model_client = ModelClient("", mock)
            expect(model_client.host) is not None
            pipeline_client = PipelineClient("", mock)
            expect(pipeline_client.host) is not None

        def when_set_correct_type_url(expect):
            mgmt_instance = InstillInstance(
                mgmt_service.MgmtPublicServiceStub,
                "test_url",
                "token",
                False,
                False,
            )
            mgmt_client = MgmtClient("")
            mgmt_client.host = mgmt_instance
            expect(mgmt_client.host.url) == "test_url"

            model_instance = InstillInstance(
                model_service.ModelPublicServiceStub,
                "test_url",
                "token",
                False,
                False,
            )
            model_client = ModelClient("", mock)
            model_client.host = model_instance
            expect(model_client.host.url) == "test_url"

            pipeline_instance = InstillInstance(
                pipeline_service.PipelinePublicServiceStub,
                "test_url",
                "token",
                False,
                False,
            )
            pipeline_client = PipelineClient("", mock)
            pipeline_client.host = pipeline_instance
            expect(pipeline_client.host.url) == "test_url"

        def when_set_correct_type_token(expect):
            mgmt_instance = InstillInstance(
                mgmt_service.MgmtPublicServiceStub,
                "test_url",
                "token",
                False,
                False,
            )
            mgmt_client = MgmtClient("")
            mgmt_client.host = mgmt_instance
            expect(mgmt_client.host.token) == "token"

            model_instance = InstillInstance(
                model_service.ModelPublicServiceStub,
                "test_url",
                "token",
                False,
                False,
            )
            model_client = ModelClient("", mock)
            model_client.host = model_instance
            expect(model_client.host.token) == "token"

            pipeline_instance = InstillInstance(
                pipeline_service.PipelinePublicServiceStub,
                "test_url",
                "token",
                False,
                False,
            )
            pipeline_client = PipelineClient("", mock)
            pipeline_client.host = pipeline_instance
            expect(pipeline_client.host.token) == "token"
